@@ -1,6 +1,6 @@
 /* Cepstrumc.c
  *
- * Copyright (C) 1994-2017 David Weenink
+ * Copyright (C) 1994-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,9 +58,9 @@ void structCepstrumc :: v_info () {
 	MelderInfo_writeLine (U"  Number of coefficients: ", maxnCoefficients);
 }
 
-void Cepstrumc_Frame_init (Cepstrumc_Frame me, int nCoefficients) {
-	my c = NUMvector<double> (0, nCoefficients);
-	my nCoefficients = nCoefficients;
+void Cepstrumc_Frame_init (Cepstrumc_Frame me, integer nCoefficients) {
+	my c = newVECzero (nCoefficients);
+	my nCoefficients = my c.size; // maintain invariant
 }
 
 void Cepstrumc_init (Cepstrumc me, double tmin, double tmax, integer nt, double dt, double t1, integer nCoefficients, double samplingFrequency) {
