@@ -1,6 +1,6 @@
 /* Sound_and_LPC.cpp
  *
- * Copyright (C) 1994-2019 David Weenink
+ * Copyright (C) 1994-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,10 @@
 #include "Sound_extensions.h"
 #include "Vector.h"
 #include "Spectrum.h"
+#include <thread>
+#include <atomic>
+#include <functional>
+#include <vector>
 #include "NUM2.h"
 
 #define LPC_METHOD_AUTO 1
@@ -55,30 +59,37 @@ static void LPC_Frame_Sound_filter (LPC_Frame me, Sound thee, integer channel) {
 	}
 }
 
-void LPC_Frame_Sound_filterInverse (LPC_Frame me, Sound thee, integer channel) {
-	autoVEC y = newVECzero (my nCoefficients);
-	for (integer i = 1; i <= thy nx; i ++) {
-		const double y0 = thy z [channel] [i];
-		for (integer j = 1; j <= my nCoefficients; j ++)
-			thy z [channel] [i] += my a [j] * y [j];
-		for (integer j = my nCoefficients; j > 1; j --)
-			y [j] = y [j - 1];
-		y [1] = y0;
-	}
+static integer getLPCAnalysisWorkspaceSize (integer numberOfSamples, integer numberOfCoefficients, kLPC_Analysis method) {
+	integer size = 0;
+	if (method == kLPC_Analysis :: AUTOCORRELATION)
+		size = 3 * numberOfCoefficients + 2;
+	else if (method == kLPC_Analysis :: COVARIANCE)
+		size = numberOfCoefficients * (numberOfCoefficients + 1) / 2 + 4 * numberOfCoefficients + 2; 
+	else if (method == kLPC_Analysis :: BURG)
+		size = 3 * numberOfSamples;
+	else if (method == kLPC_Analysis :: MARPLE)
+		size = 3 * (numberOfCoefficients + 1);
+	return size;
 }
 
-static int Sound_into_LPC_Frame_auto (Sound me, LPC_Frame thee) {
-	const integer numberOfCoefficients = thy nCoefficients;
+static autoVEC getLPCAnalysisWorkspace (integer numberOfSamples, integer numberOfCoefficients, kLPC_Analysis method) {
+	integer size = getLPCAnalysisWorkspaceSize (numberOfSamples, numberOfCoefficients, method);
+	autoVEC result = newVECraw (size);
+	return result;
+}
 
-	autoVEC r = newVECzero (numberOfCoefficients + 1);
-	autoVEC a = newVECzero (numberOfCoefficients + 1);
-	autoVEC rc = newVECzero (numberOfCoefficients);
+static int Sound_into_LPC_Frame_auto (Sound me, LPC_Frame thee, VEC const& workspace) {
+	Melder_assert (thy nCoefficients == thy a.size); // check invariant
+	const integer numberOfCoefficients = thy nCoefficients, np1 = numberOfCoefficients + 1;
 
-	const VEC x = my z.row (1);
+	//workspace <<= 0.0; not necessary !
+	VEC r = workspace. part (1, np1); // autoVEC r = newVECzero (numberOfCoefficients + 1);
+	VEC a = workspace. part (np1 + 1, 2 * np1); // autoVEC a = newVECzero (numberOfCoefficients + 1);
+	VEC rc = workspace. part (2 * np1 + 1, 2 * np1 + numberOfCoefficients); // autoVEC rc = newVECzero (numberOfCoefficients);
+	const VECVU x = my z.row (1);
 	integer i = 1; // For error condition at end
 	for (i = 1; i <= numberOfCoefficients + 1; i ++)
-		for (integer j = 1; j <= my nx - i + 1; j ++)
-			r [i] += x [j] * x [j + i - 1];
+		r [i] = NUMinner (x.part (1, my nx - i + 1), x.part (i, my nx));
 	if (r [1] == 0.0) {
 		i = 1; // !
 		goto end;
@@ -98,41 +109,40 @@ static int Sound_into_LPC_Frame_auto (Sound me, LPC_Frame thee) {
 		}
 		a [i + 1] = rc [i];
         thy gain += rc [i] * s;
-		if (thy gain <= 0)
+		if (thy gain <= 0.0)
 			goto end;
 	}
 end:
 	i--;
 	for (integer j = 1; j <= i; j ++)
 		thy a [j] = a [j + 1];
-
 	if (i == numberOfCoefficients)
 		return 1;
-
-	thy nCoefficients = i;
-	for (integer j = i + 1; j <= numberOfCoefficients; j ++)
-		thy a [j] = 0.0;
+	thy a.resize (i);
+	thy nCoefficients = thy a.size; // maintain invariant
 	return 0; // Melder_warning ("Fewer coefficients than asked for.");
 }
 
-/* Markel&Gray, LP of S, page 221
+/*
+	Markel&Gray, LP of S, page 221
 	work [1..m(m+1)/2+m+m+1+m+m+1]
-	b = & work [1]
-	grc = & work [m*(m+1)/2+1];
-	a = & work [m*(m+1)/2+m+1];
-	beta = & work [m+1)/2+m+m+1+1];
-	cc = & work [m+1)/2+m+m+1+m+1]
-	for (i=1; i<=m(m+1)/2+m+m+1+m+m+1;i ++) work [i] = 0;
 */
-static int Sound_into_LPC_Frame_covar (Sound me, LPC_Frame thee) {
+static int Sound_into_LPC_Frame_covar (Sound me, LPC_Frame thee, VEC const& workspace) {
+	Melder_assert (thy nCoefficients == thy a.size); // check invariant
 	const integer n = my nx, m = thy nCoefficients;
 	constVEC x = my z.row (1);
-
-	autoVEC b = newVECzero (m * (m + 1) / 2);
-	autoVEC grc = newVECzero (m);
-	autoVEC a = newVECzero (m + 1);
-	autoVEC beta = newVECzero (m);
-	autoVEC cc = newVECzero (m + 1);
+	
+	workspace <<= 0.0;
+	integer start = 1, end = m * (m + 1) / 2;
+	VEC b = workspace. part (start, end); // autoVEC b = newVECzero (m * (m + 1) / 2);
+	start = end + 1; end += m;
+	VEC grc = workspace. part (start, end); //autoVEC grc = newVECzero (m);
+	start = end + 1; end += m;
+	VEC beta = workspace. part (start, end); // autoVEC beta = newVECzero (m);
+	start = end + 1; end += m + 1;
+	VEC a = workspace. part (start, end); // autoVEC a = newVECzero (m + 1);
+	start = end + 1; end += m + 1;
+	VEC cc = workspace. part (start, end); // autoVEC cc = newVECzero (m + 1);
 
 	thy gain = 0.0;
 	integer i;
@@ -158,7 +168,7 @@ static int Sound_into_LPC_Frame_covar (Sound me, LPC_Frame thee) {
 		for (integer j = 1; j <= i; j ++)
 			cc [i - j + 2] = cc [i - j + 1] + x [m - i + 1] * x [m - i + j] - x [n - i + 1] * x [n - i + j];
 
-		cc [1] = 0.0;
+		cc [1] = 0.0; // TODO NUMinner
 		for (integer j = m + 1; j <= n; j ++)
 			cc [1] += x [j - i] * x [j]; // 30
 
@@ -198,39 +208,107 @@ static int Sound_into_LPC_Frame_covar (Sound me, LPC_Frame thee) {
 	}
 end:
 	i--;
-
 	for (integer j = 1; j <= i; j ++)
 		thy a [j] = a [j + 1];
 	if (i == m)
 		return 1;
-
-	thy nCoefficients = i;
-    thy a.part (i + 1, m) <<= 0.0;
+	thy a.resize (i);
+	thy nCoefficients = thy a.size;
 	return 0; // Melder_warning ("Fewer coefficients than asked for.");
 }
 
-static int Sound_into_LPC_Frame_burg (Sound me, LPC_Frame thee) {
-	thy gain = VECburg (thy a.get(), my z.row(1));
+static double VECburg_buffered (VEC const& a, constVEC const& x, VEC const& workspace) {
+	const integer n = x.size, m = a.size;
+	for (integer j = 1; j <= m; j ++)
+		a [j] = 0.0;
+
+	VEC b1 = workspace. part (1, n); // autoVEC b1 = newVECzero (n);
+	VEC b2 = workspace. part (n + 1, 2 * n); // autoVEC b2 = newVECzero (n);
+	VEC aa = workspace. part (2 * n + 1, 2 * n + m); // autoVEC aa = newVECzero (m);
+
+	// (3)
+
+	longdouble p = 0.0;
+	for (integer j = 1; j <= n; j ++)
+		p += x [j] * x [j];
+
+	longdouble xms = p / n;
+	if (xms <= 0.0) {
+		return xms;	// warning empty
+	}
+	// (9)
+
+	b1 [1] = x [1];
+	b2 [n - 1] = x [n];
+	for (integer j = 2; j <= n - 1; j ++)
+		b1 [j] = b2 [j - 1] = x [j];
+
+	for (integer i = 1; i <= m; i ++) {
+		// (7)
+
+		longdouble num = 0.0, denum = 0.0;
+		for (integer j = 1; j <= n - i; j ++) {
+			num += b1 [j] * b2 [j];
+			denum += b1 [j] * b1 [j] + b2 [j] * b2 [j];
+		}
+
+		if (denum <= 0.0)
+			return 0.0;	// warning ill-conditioned
+
+		a [i] = 2.0 * num / denum;
+
+		// (10)
+
+		xms *= 1.0 - a [i] * a [i];
+
+		// (5)
+
+		for (integer j = 1; j <= i - 1; j ++)
+			a [j] = aa [j] - a [i] * aa [i - j];
+
+		if (i < m) {
+
+			// (8) Watch out: i -> i+1
+
+			for (integer j = 1; j <= i; j ++)
+				aa [j] = a [j];
+			for (integer j = 1; j <= n - i - 1; j ++) {
+				b1 [j] -= aa [i] * b2 [j];
+				b2 [j] = b2 [j + 1] - aa [i] * b1 [j + 1];
+			}
+		}
+	}
+	return xms;
+}
+
+static int Sound_into_LPC_Frame_burg (Sound me, LPC_Frame thee, VEC const& workspace) {
+	Melder_assert (thy nCoefficients == thy a.size); // check invariant
+	thy gain = VECburg_buffered (thy a.get(), my z.row(1), workspace);
+	if (thy gain <= 0.0) {
+		thy a.resize (0);
+		thy nCoefficients = thy a.size; // maintain invariant
+		return 0;
+	}
 	thy gain *= my nx;
 	for (integer i = 1; i <= thy nCoefficients; i ++)
 		thy a [i] = -thy a [i];
 	return thy gain != 0.0;
 }
 
-static int Sound_into_LPC_Frame_marple (Sound me, LPC_Frame thee, double tol1, double tol2) {
-	const integer n = my nx, mmax = thy nCoefficients;
+static int Sound_into_LPC_Frame_marple (Sound me, LPC_Frame thee, double tol1, double tol2, VEC const& workspace) {
+	const integer n = my nx, mmax = thy nCoefficients, mmaxp1 = mmax + 1;
 	int status = 1;
+	// workspace.all () << 0.0 not necessary
 	constVEC x = my z.row (1);
-
-	autoVEC c = newVECzero (mmax + 1);
-	autoVEC d = newVECzero (mmax + 1);
-	autoVEC r = newVECzero (mmax + 1);
+	VEC c = workspace .part (1, mmaxp1); // autoVEC c = newVECzero (mmax + 1);
+	VEC d = workspace .part (mmaxp1 + 1, 2 * mmaxp1); // autoVEC d = newVECzero (mmax + 1);
+	VEC r = workspace .part (2 * mmaxp1 + 1, 3 * mmaxp1); // autoVEC r = newVECzero (mmax + 1);
 	double e0 = 2.0 * NUMsum2 (x);
 	integer m = 1;
 	if (e0 == 0.0) {
-		m = 0;
-		thy gain *= 0.5; // because e0 is twice the energy
-		thy nCoefficients = m;
+		thy a.resize (0);
+		thy nCoefficients = thy a.size; // maintain invariant
+		thy gain = 0.0;
 		return 0; // warning no signal
 	}
 	double q1 = 1.0 / e0;
@@ -351,11 +429,12 @@ static int Sound_into_LPC_Frame_marple (Sound me, LPC_Frame thee, double tol1, d
 	}
 end:
 	thy gain *= 0.5; // because e0 is twice the energy
-	thy nCoefficients = m;
+	thy a.resize (m);
+	thy nCoefficients = thy a.size; // maintain invariant
 	return status == 1 || status == 4 || status == 5;
 }
 
-static autoLPC _Sound_to_LPC (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency, int method, double tol1, double tol2) {
+static autoLPC Sound_to_LPC_noThreads (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency, kLPC_Analysis method, double tol1, double tol2) {
 	const double samplingFrequency = 1.0 / my dx;
 	double windowDuration = 2.0 * analysisWidth; // Gaussian window
 	Melder_require (Melder_roundDown (windowDuration / my dx) > predictionOrder,
@@ -373,7 +452,12 @@ static autoLPC _Sound_to_LPC (Sound me, int predictionOrder, double analysisWidt
 	autoSound sframe = Sound_createSimple (1, windowDuration, samplingFrequency);
 	autoSound window = Sound_createGaussian (windowDuration, samplingFrequency);
 	autoLPC thee = LPC_create (my xmin, my xmax, numberOfFrames, dt, t1, predictionOrder, my dx);
+	for (integer iframe = 1; iframe <= numberOfFrames; iframe ++) {
+		const LPC_Frame lpcFrame = & thy d_frames [iframe];
+		LPC_Frame_init (lpcFrame, predictionOrder);
+	}
 
+	autoVEC workspace = getLPCAnalysisWorkspace (sframe -> nx, predictionOrder, method);
 	autoMelderProgress progress (U"LPC analysis");
 
 	if (preEmphasisFrequency < samplingFrequency / 2.0)
@@ -382,41 +466,133 @@ static autoLPC _Sound_to_LPC (Sound me, int predictionOrder, double analysisWidt
 	for (integer iframe = 1; iframe <= numberOfFrames; iframe ++) {
 		const LPC_Frame lpcframe = & thy d_frames [iframe];
 		const double t = Sampled_indexToX (thee.get(), iframe);
-		LPC_Frame_init (lpcframe, predictionOrder);
-		Sound_into_Sound (sound.get(), sframe.get(), t - windowDuration / 2);
+		Sound_into_Sound (sound.get(), sframe.get(), t - windowDuration / 2.0);
 		Vector_subtractMean (sframe.get());
 		Sounds_multiply (sframe.get(), window.get());
-		if (method == LPC_METHOD_AUTO) {
-			if (! Sound_into_LPC_Frame_auto (sframe.get(), lpcframe))
-				frameErrorCount ++;
-		} else if (method == LPC_METHOD_COVAR) {
-			if (! Sound_into_LPC_Frame_covar (sframe.get(), lpcframe))
-				frameErrorCount ++;
-		} else if (method == LPC_METHOD_BURG) {
-			if (! Sound_into_LPC_Frame_burg (sframe.get(), lpcframe))
-				frameErrorCount ++;
-		} else if (method == LPC_METHOD_MARPLE) {
-			if (! Sound_into_LPC_Frame_marple (sframe.get(), lpcframe, tol1, tol2))
-				frameErrorCount ++;
-		}
+		integer status = 1;
+		if (method == kLPC_Analysis :: AUTOCORRELATION)
+			status = Sound_into_LPC_Frame_auto (sframe.get(), lpcframe, workspace.get());
+		else if (method == kLPC_Analysis :: COVARIANCE)
+			status = Sound_into_LPC_Frame_covar (sframe.get(), lpcframe, workspace.get());
+		else if (method == kLPC_Analysis :: BURG)
+			status = Sound_into_LPC_Frame_burg (sframe.get(), lpcframe, workspace.get());
+		else if (method == kLPC_Analysis :: MARPLE)
+			status = Sound_into_LPC_Frame_marple (sframe.get(), lpcframe, tol1, tol2, workspace.get());
+		if (status != 0)
+			frameErrorCount ++;
+	
 		if (iframe % 10 == 1)
 			Melder_progress ( (double) iframe / numberOfFrames, U"LPC analysis of frame ", iframe, U" out of ", numberOfFrames, U".");
 	}
 	return thee;
 }
 
-autoLPC Sound_to_LPC_auto (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency) {
+static autoLPC Sound_to_LPC (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency, kLPC_Analysis method, double tol1, double tol2) {
+	const integer numberOfProcessors = std::thread::hardware_concurrency ();
+	if (numberOfProcessors <= 1) {
+		/*
+			We cannot use multithreading.
+		*/
+		return Sound_to_LPC_noThreads (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, method, tol1, tol2);
+	}
+	const double samplingFrequency = 1.0 / my dx;
+	double windowDuration = 2.0 * analysisWidth; // Gaussian window
+	Melder_require (Melder_roundDown (windowDuration / my dx) > predictionOrder,
+		U"Analysis window duration too short.\n For a prediction order of ", predictionOrder,
+		U" the analysis window duration should be greater than ", my dx * (predictionOrder + 1), U"Please increase the analysis window duration or lower the prediction order.");
+	
+	if (windowDuration > my dx * my nx) {
+		windowDuration = my dx * my nx;
+	}
+	double t1;
+	integer numberOfFrames;
+	Sampled_shortTermAnalysis (me, windowDuration, dt, & numberOfFrames, & t1);
+	autoSound sound = Data_copy (me);
+	autoSound window = Sound_createGaussian (windowDuration, samplingFrequency);
+	autoLPC thee = LPC_create (my xmin, my xmax, numberOfFrames, dt, t1, predictionOrder, my dx);
+	/*
+		Because of threading we initialise the frames beforehand.
+		We initialize the coefficient vector with a size equal to the prediction order.
+	*/
+	for (integer iframe = 1; iframe <= numberOfFrames; iframe ++) {
+		const LPC_Frame lpcFrame = & thy d_frames [iframe];
+		LPC_Frame_init (lpcFrame, predictionOrder);
+	}
+	if (preEmphasisFrequency < samplingFrequency / 2.0)
+		Sound_preEmphasis (sound.get(), preEmphasisFrequency);
+	
+	constexpr integer maximumNumberOfThreads = 16;
+	integer numberOfThreads, numberOfFramesPerThread = 25;
+	NUMgetThreadingInfo (numberOfFrames, std::min (numberOfProcessors, maximumNumberOfThreads), & numberOfFramesPerThread, & numberOfThreads);
+	/*
+		We have to reserve all the needed working memory for each thread beforehand.
+	*/
+	autoSound sframe [maximumNumberOfThreads + 1];
+	for (integer ithread = 1; ithread <= numberOfThreads; ithread ++)
+		sframe [ithread] = Sound_createSimple (1, windowDuration, samplingFrequency);
+	
+	const integer worspaceSize = getLPCAnalysisWorkspaceSize (sframe [1] -> nx, predictionOrder, method);
+	Melder_require (worspaceSize > 0,
+		U"The workspace size is not properly defined.");
+	autoMAT workspace = newMATraw (numberOfThreads, worspaceSize);
+
+	std::vector <std::thread> thread (numberOfThreads);
+	std::atomic<integer> frameErrorCount (0);
+	
 	try {
-		autoLPC thee = _Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, LPC_METHOD_AUTO, 0.0, 0.0);
+		for (integer ithread = 1; ithread <= numberOfThreads; ithread ++) {
+			Sound soundFrame = sframe [ithread]. get(), fullsound = sound.get(), windowFrame = window.get();
+			VEC threadWorkspace = workspace. row (ithread);
+			LPC lpc = thee.get();
+			const integer firstFrame = 1 + (ithread - 1) * numberOfFramesPerThread;
+			const integer lastFrame = ( ithread == numberOfThreads ? numberOfFrames : firstFrame + numberOfFramesPerThread - 1 );
+			
+			thread [ithread - 1] = std::thread ([=, & frameErrorCount]() {
+				for (integer iframe = firstFrame; iframe <= lastFrame; iframe ++) {
+					const LPC_Frame lpcframe = & lpc -> d_frames [iframe];
+					const double t = Sampled_indexToX (lpc, iframe);
+					Sound_into_Sound (fullsound, soundFrame, t - windowDuration / 2.0);
+					Vector_subtractMean (soundFrame);
+					Sounds_multiply (soundFrame, windowFrame);
+					integer status = 1;
+					if (method == kLPC_Analysis :: AUTOCORRELATION)
+						status = Sound_into_LPC_Frame_auto (soundFrame, lpcframe, threadWorkspace);
+					else if (method == kLPC_Analysis :: COVARIANCE)
+						status = Sound_into_LPC_Frame_covar (soundFrame, lpcframe, threadWorkspace);
+					else if (method == kLPC_Analysis :: BURG)
+						status = Sound_into_LPC_Frame_burg (soundFrame, lpcframe, threadWorkspace);
+					else if (method == kLPC_Analysis :: MARPLE)
+						status = Sound_into_LPC_Frame_marple (soundFrame, lpcframe, tol1, tol2, threadWorkspace);
+					if (status != 0)
+						++ frameErrorCount;
+				}
+			});
+		}
+	} catch (MelderError) {
+		for (integer ithread = 1; ithread <= numberOfThreads; ithread ++) {
+			if (thread [ithread - 1]. joinable ())
+				thread [ithread - 1]. join ();
+		}
+		throw;
+	}
+	for (integer ithread = 1; ithread <= numberOfThreads; ithread ++)
+		thread [ithread - 1]. join ();
+	
+	return thee;
+}
+
+autoLPC Sound_to_LPC_autocorrelation (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency) {
+	try {
+		autoLPC thee = Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, kLPC_Analysis :: AUTOCORRELATION, 0.0, 0.0);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no LPC (auto) created.");
 	}
 }
 
-autoLPC Sound_to_LPC_covar (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency) {
+autoLPC Sound_to_LPC_covariance (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency) {
 	try {
-		autoLPC thee = _Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, LPC_METHOD_COVAR, 0.0, 0.0);
+		autoLPC thee = Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, kLPC_Analysis :: COVARIANCE, 0.0, 0.0);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no LPC (covar) created.");
@@ -425,7 +601,7 @@ autoLPC Sound_to_LPC_covar (Sound me, int predictionOrder, double analysisWidth,
 
 autoLPC Sound_to_LPC_burg (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency) {
 	try {
-		autoLPC thee = _Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, LPC_METHOD_BURG, 0.0, 0.0);
+		autoLPC thee = Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, kLPC_Analysis :: BURG, 0.0, 0.0);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no LPC (burg) created.");
@@ -434,7 +610,7 @@ autoLPC Sound_to_LPC_burg (Sound me, int predictionOrder, double analysisWidth,
 
 autoLPC Sound_to_LPC_marple (Sound me, int predictionOrder, double analysisWidth, double dt, double preEmphasisFrequency, double tol1, double tol2) {
 	try {
-		autoLPC thee = _Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, LPC_METHOD_MARPLE, tol1, tol2);
+		autoLPC thee = Sound_to_LPC (me, predictionOrder, analysisWidth, dt, preEmphasisFrequency, kLPC_Analysis :: MARPLE, tol1, tol2);
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": no LPC (marple) created.");
@@ -449,12 +625,13 @@ autoSound LPC_Sound_filterInverse (LPC me, Sound thee) {
 			U"The domains of LPC and Sound should be equal.");
 		
 		autoSound him = Data_copy (thee);
-
+		VEC source = his z.row (1);
+		VEC sound = thy z.row (1);
 		for (integer isamp = 1; isamp <= his nx; isamp ++) {
 			const double sampleTime = Sampled_indexToX (him.get(), isamp);
 			const integer frameNumber = Sampled_xToNearestIndex (me, sampleTime);
 			if (frameNumber < 1 || frameNumber > my nx) {
-				his z [1] [isamp] = 0.0;
+				source [isamp] = 0.0;
 				continue;
 			}
 			const LPC_Frame frame = & my d_frames [frameNumber];
@@ -462,7 +639,7 @@ autoSound LPC_Sound_filterInverse (LPC me, Sound thee) {
 			const integer maximumSoundDepth = isamp - 1;
 			const integer usableDepth = std::min (maximumFilterDepth, maximumSoundDepth);
 			for (integer icoef = 1; icoef <= usableDepth; icoef ++)
-				his z [1] [isamp] += frame -> a [icoef] * thy z [1] [isamp - icoef];
+				source [isamp] += frame -> a [icoef] * sound [isamp - icoef];
 		}
 		return him;
 	} catch (MelderError) {
@@ -573,11 +750,13 @@ void LPC_Sound_filterInverseWithFilterAtTime_inplace (LPC me, Sound thee, intege
 		Melder_clip (1_integer, & frameIndex, my nx);   // constant extrapolation
 		if (channel > thy ny)
 			channel = 1;
+		LPC_Frame lpc = & my d_frames [frameIndex];
+		autoVEC work = newVECraw (lpc -> nCoefficients);
 		if (channel > 0)
-			LPC_Frame_Sound_filterInverse (& (my d_frames [frameIndex]), thee, channel);
+			VECfilterInverse_inplace (thy z.row (channel), lpc -> a.get(), work);
 		else
 			for (integer ichan = 1; ichan <= thy ny; ichan ++)
-				LPC_Frame_Sound_filterInverse (& (my d_frames [frameIndex]), thee, ichan);
+				VECfilterInverse_inplace (thy z.row (ichan), lpc -> a.get(), work);
 	} catch (MelderError) {
 		Melder_throw (thee, U": not inverse filtered.");
 	}
