@@ -1,6 +1,6 @@
 /* praat_LPC_init.cpp
  *
- * Copyright (C) 1994-2019 David Weenink
+ * Copyright (C) 1994-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@
 #include "DTW.h"
 #include "FilterBank.h"
 #include "Formant_extensions.h"
+#include "FormantPath.h"
+#include "FormantPathEditor.h"
 #include "LPC.h"
 #include "MFCC.h"
 #include "LFCC.h"
@@ -45,6 +47,8 @@
 #include "LPC_to_Spectrogram.h"
 #include "LPC_to_Spectrum.h"
 #include "NUM2.h"
+#include "PowerCepstrum.h"
+#include "PowerCepstrogram.h"
 #include "Sound_and_LPC.h"
 #include "Sound_and_LPC_robust.h"
 #include "Sound_and_Cepstrum.h"
@@ -65,6 +69,78 @@ static const conststring32 MODIFY_BUTTON   = U"Modify -";
 void praat_CC_init (ClassInfo klas);
 void praat_TimeFrameSampled_query_init (ClassInfo klas);
 
+static void cb_FormantPathEditor_publication (Editor /* editor */, autoDaata publication) {
+	/*
+	 * Keep the gate for error handling.
+	 */
+	try {
+		praat_new (publication.move());
+		praat_updateSelection ();
+	} catch (MelderError) {
+		Melder_flushError ();
+	}
+}
+
+DIRECT (WINDOW_FormantPath_viewAndEdit) {
+	if (theCurrentPraatApplication -> batch)
+		Melder_throw (U"Cannot view or edit a Formant from batch.");
+	FIND_ONE_WITH_IOBJECT (FormantPath)
+		autoFormantPathEditor editor = FormantPathEditor_create (ID_AND_FULL_NAME, me);
+		Editor_setPublicationCallback (editor.get(), cb_FormantPathEditor_publication);
+		praat_installEditor (editor.get(), IOBJECT);
+		editor.releaseToUser();
+	END
+}
+FORM (MODIFY_FormantPath_setNavigationContext, U"FormantPath: Set navigation context", nullptr) {
+	OPTIONMENU_ENUM (kContextCombination, contextCombination, U"Context combination", kContextCombination::DEFAULT)
+	BOOLEAN (matchContextOnly, U"Match context only", false)
+	OK
+DO
+	MODIFY_EACH (FormantPath)
+		FormantPath_setNavigationContext (me, contextCombination, matchContextOnly);
+	MODIFY_EACH_END
+}
+
+DIRECT (NEW_FormantPath_extractFormant) {
+	CONVERT_EACH (FormantPath)
+		autoFormant result = FormantPath_extractFormant (me);
+	CONVERT_EACH_END (my name.get())
+}
+
+DIRECT (NEW_FormantPath_extractTextGrid) {
+	CONVERT_EACH (FormantPath)
+		autoTextGrid result = FormantPath_extractTextGrid (me);
+	CONVERT_EACH_END (my name.get())
+}
+
+FORM (MODIFY_FormantPath_setNavigationLabels, U"FormantPath: Set navigation labels", nullptr) {
+	NATURAL (intervalTierNumber, U"Interval tier number", U"1")
+	OPTIONMENU_ENUM (kMelder_string, criterion, U"Matching criterion", kMelder_string::DEFAULT)
+	OK
+DO
+	MODIFY_FIRST_OF_TWO (FormantPath, Strings)	
+		FormantPath_setNavigationLabels (me, you, intervalTierNumber, criterion);
+	MODIFY_FIRST_OF_TWO_END
+}
+
+FORM (MODIFY_FormantPath_setLeftContextNavigationLabels, U"FormantPath: Set left context navigation labels", nullptr) {
+	OPTIONMENU_ENUM (kMelder_string, criterion, U"Matching criterion", kMelder_string::DEFAULT)
+	OK
+DO
+	MODIFY_FIRST_OF_TWO (FormantPath, Strings)
+		FormantPath_setLeftContextNavigationLabels (me, you, criterion);
+	MODIFY_FIRST_OF_TWO_END
+}
+
+FORM (MODIFY_FormantPath_setRightContextNavigationLabels, U"FormantPath: Set right context navigation labels", nullptr) {
+	OPTIONMENU_ENUM (kMelder_string, criterion, U"Matching criterion", kMelder_string::DEFAULT)
+	OK
+DO
+	MODIFY_FIRST_OF_TWO (FormantPath, Strings)	
+		FormantPath_setRightContextNavigationLabels (me, you, criterion);
+	MODIFY_FIRST_OF_TWO_END
+}
+
 /********************** Cepstrum  ****************************************/
 
 DIRECT (NEW_Cepstrum_downto_PowerCepstrum) {
@@ -193,7 +269,7 @@ DO
 	NUMBER_ONE (PowerCepstrum)
 		double result;
 		PowerCepstrum_fitTrendLine (me, fromQuefrency_trendLine, toQuefrency_trendLine, & result, nullptr, lineType, fitMethod);
-	NUMBER_ONE_END (U" dB / ", lineType == kCepstrumTrendType::Linear ? U"s" : U"ln (s)");
+	NUMBER_ONE_END (U" dB / ", lineType == kCepstrumTrendType::LINEAR ? U"s" : U"ln (s)");
 }
 
 FORM (REAL_PowerCepstrum_getTrendLineIntercept, U"PowerCepstrum: Get trend line intercept", U"PowerCepstrum: Get trend line intercept...") {
@@ -582,6 +658,31 @@ DO
 	GRAPHICS_EACH_END
 }
 
+FORM (INTEGER_LineSpectralFrequencies_getNumberOfFrequencies, U"LineSpectralFrequencies: Get number of frequencies", U"LineSpectralFrequencies: Get number of frequencies...") {
+	NATURAL (frameNumber, U"Frame number", U"2")
+	OK
+DO
+	INTEGER_ONE (LineSpectralFrequencies)
+		my checkIndex (frameNumber);
+		const integer result = my d_frames [frameNumber]. numberOfFrequencies;
+	INTEGER_ONE_END (U" frequencies")
+}
+
+FORM (NUMVEC_LineSpectralFrequencies_listFrequenciesInFrame, U"LineSpectralFrequencies: List all frequencies in frame", U"") {
+	NATURAL (frameNumber, U"Frame number", U"10")
+	OK
+DO
+	NUMVEC_ONE (LineSpectralFrequencies)
+		autoVEC result = LineSpectralFrequencies_listFrequenciesInFrame (me, frameNumber);
+	NUMVEC_ONE_END
+}
+
+DIRECT (NUMMAT_LineSpectralFrequencies_listAllFrequencies) {
+	NUMMAT_ONE (LineSpectralFrequencies)
+		autoMAT result = LineSpectralFrequencies_listAllFrequencies (me);
+	NUMMAT_ONE_END	
+}
+
 DIRECT (NEW_LineSpectralFrequencies_to_LPC) {
 	CONVERT_EACH (LineSpectralFrequencies)
 		autoLPC result = LineSpectralFrequencies_to_LPC (me);
@@ -617,12 +718,43 @@ FORM (INTEGER_LPC_getNumberOfCoefficients, U"LPC: Get number of coefficients", U
 	OK
 DO
 	INTEGER_ONE (LPC)
-		Melder_require (frameNumber <= my nx,
-			U"Your frame number (", frameNumber, U") is too large. It should be between 1 and ", my nx, U".");
+		my checkIndex (frameNumber);
 		const integer result = my d_frames [frameNumber]. nCoefficients;
 	INTEGER_ONE_END (U" coefficients")
 }
 
+FORM (NUMVEC_LPC_listAllCoefficientsInFrame, U"", U"") {
+	NATURAL (frameNumber, U"Frame number", U"10")
+	OK
+DO
+	NUMVEC_ONE (LPC)
+		autoVEC result = LPC_listCoefficientsInFrame (me, frameNumber);
+	NUMVEC_ONE_END
+}
+
+DIRECT (NUMMAT_LPC_listAllCoefficients) {
+	NUMMAT_ONE (LPC)
+		autoMAT result = LPC_listAllCoefficients (me);
+	NUMMAT_ONE_END	
+}
+
+FORM (REAL_LPC_getGainInFrame, U"LPC: Get gain in frame", U"LPC: Get gain in frame...") {
+	NATURAL (frameNumber, U"Frame number", U"10")
+	OK
+DO
+	NUMBER_ONE (LPC)
+		double result = undefined;
+		if (frameNumber > 0 && frameNumber <= my nx)
+			result = my d_frames [frameNumber] .gain;
+	NUMBER_ONE_END (U" gain in frame ", frameNumber)
+}
+
+DIRECT (NUMVEC_LPC_listAllGains) {
+	NUMVEC_ONE (LPC)
+		autoVEC result = LPC_listAllGains (me);
+	NUMVEC_ONE_END
+}
+
 FORM (GRAPHICS_LPC_drawPoles, U"LPC: Draw poles", U"LPC: Draw poles...") {
 	REAL (time, U"Time (seconds)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
@@ -777,6 +909,70 @@ DO
 	CONVERT_EACH_END (my name.get())
 }
 
+FORM (NEW_Sound_to_FormantPath, U"Sound: To FormantPath", nullptr) {
+	REAL (timeStep, U"Time step (s)", U"0.005")
+	POSITIVE (maximumNumberOfFormants, U"Max. number of formants", U"5.0")
+	REAL (maximumFormantFrequency, U"Maximum formant (Hz)", U"5500.0 (= adult female)")
+	POSITIVE (windowLength, U"Window length (s)", U"0.025")
+	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
+	OPTIONMENU_ENUM (kLPC_Analysis, lpcModel, U"LPC model", kLPC_Analysis::DEFAULT)
+	LABEL (U"The minimum and maximum ceilings are determined as:")
+	LABEL (U" maximumFormant +/- numberOfSteps * ceilingStep")
+	POSITIVE (ceilingStep, U"Ceiling step size (Hz)", U"250.0")
+	NATURAL (numberOfStepsToACeiling, U"Number of steps up / down", U"4")
+	LABEL (U"For Marple analysis:")
+	POSITIVE (marple_tol1, U"Tolerance 1", U"1e-6")
+	POSITIVE (marple_tol2, U"Tolerance 2", U"1e-6")
+	LABEL (U"For Robust analysis:")
+	POSITIVE (huber_numberOfStdDev, U"Number of std. dev.", U"1.5")
+	NATURAL (huber_maximumNumberOfIterations, U"Maximum number of iterations", U"5")
+	REAL (huber_tolerance, U"Tolerance", U"0.000001")
+	BOOLEAN (sourcesAsMultichannel, U"Get sources as multi channel sound", false)
+	OK
+DO
+	CONVERT_EACH (Sound)
+		autoSound multichannel;
+		autoFormantPath result = Sound_to_FormantPath_any (me, lpcModel, timeStep, maximumNumberOfFormants, maximumFormantFrequency, windowLength, preEmphasisFrequency, ceilingStep, numberOfStepsToACeiling, marple_tol1, marple_tol2, huber_numberOfStdDev, huber_tolerance, huber_maximumNumberOfIterations,
+			( sourcesAsMultichannel ? & multichannel : nullptr ));
+		if (sourcesAsMultichannel)
+			praat_new (multichannel.move(), my name.get(), U"_sources");
+	CONVERT_EACH_END (my name.get())
+}
+
+FORM (NEW_Sound_to_FormantPath_burg, U"Sound: To FormantPath (Burg method)", nullptr) {
+	REAL (timeStep, U"Time step (s)", U"0.005")
+	POSITIVE (maximumNumberOfFormants, U"Max. number of formants", U"5.0")
+	REAL (maximumFormantFrequency, U"Maximum formant (Hz)", U"5500.0 (= adult female)")
+	POSITIVE (windowLength, U"Window length (s)", U"0.025")
+	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
+	LABEL (U"The maximum and minimum ceilings are determined as:")
+	LABEL (U" maximumFormant +/- numberOfSteps * ceilingStep")
+	POSITIVE (ceilingStep, U"Ceiling step size (Hz)", U"250.0")
+	NATURAL (numberOfStepsToACeiling, U"Number of steps up / down", U"4")
+	OK
+DO
+	CONVERT_EACH (Sound)
+		autoFormantPath result = Sound_to_FormantPath_burg (me, timeStep, maximumNumberOfFormants, maximumFormantFrequency, windowLength, preEmphasisFrequency, ceilingStep, numberOfStepsToACeiling);
+	CONVERT_EACH_END (my name.get())	
+}
+
+FORM (NEW_Sound_and_TextGrid_to_FormantPath_burg, U"Sound & TextGrid: To FormantPath (Burg method)", U"Sound & TextGrid: To FormantPath (burg)...") {
+	REAL (timeStep, U"Time step (s)", U"0.005")
+	POSITIVE (maximumNumberOfFormants, U"Max. number of formants", U"5.0")
+	REAL (maximumFormantFrequency, U"Maximum formant (Hz)", U"5500.0 (= adult female)")
+	POSITIVE (windowLength, U"Window length (s)", U"0.025")
+	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
+	LABEL (U"The maximum and minimum ceilings are determined as:")
+	LABEL (U" maximumFormant +/- numberOfSteps * ceilingStep")
+	POSITIVE (ceilingStep, U"Ceiling step size (Hz)", U"250.0")
+	NATURAL (numberOfStepsToACeiling, U"Number of steps to up / down", U"4")
+	OK
+DO
+	CONVERT_TWO (Sound, TextGrid)
+		autoFormantPath result = Sound_and_TextGrid_to_FormantPath_burg (me, you, timeStep, maximumNumberOfFormants, maximumFormantFrequency, windowLength, preEmphasisFrequency, ceilingStep, numberOfStepsToACeiling);
+	CONVERT_TWO_END (my name.get())
+}
+
 #define Sound_to_LPC_addWarning \
 	LABEL (U"Warning 1:  for formant analysis, use \"To Formant\" instead.") \
 	LABEL (U"Warning 2:  if you do use \"To LPC\", you may want to resample first.") \
@@ -793,7 +989,7 @@ FORM (NEW_Sound_to_LPC_autocorrelation, U"Sound: To LPC (autocorrelation)", U"So
 DO
 	preEmphasisFrequency = preEmphasisFrequency < 0.0 ? 0.0 : preEmphasisFrequency;
 	CONVERT_EACH (Sound)
-		autoLPC result = Sound_to_LPC_auto (me, predictionOrder, windowLength, timeStep, preEmphasisFrequency);
+		autoLPC result = Sound_to_LPC_autocorrelation (me, predictionOrder, windowLength, timeStep, preEmphasisFrequency);
 	CONVERT_EACH_END (my name.get())
 }
 
@@ -807,7 +1003,7 @@ FORM (NEW_Sound_to_LPC_covariance, U"Sound: To LPC (covariance)", U"Sound: To LP
 DO
 	preEmphasisFrequency = preEmphasisFrequency < 0.0 ? 0.0 : preEmphasisFrequency;
 	CONVERT_EACH (Sound)
-		autoLPC result = Sound_to_LPC_covar (me, predictionOrder, windowLength, timeStep, preEmphasisFrequency);
+		autoLPC result = Sound_to_LPC_covariance (me, predictionOrder, windowLength, timeStep, preEmphasisFrequency);
 	CONVERT_EACH_END (my name.get())
 }
 
@@ -989,8 +1185,10 @@ extern void praat_TimeTier_query_init (ClassInfo klas);
 extern void praat_TimeTier_modify_init (ClassInfo klas);
 void praat_uvafon_LPC_init ();
 void praat_uvafon_LPC_init () {
-	Thing_recognizeClassesByName (classCepstrumc, classPowerCepstrum, classCepstrogram, classPowerCepstrogram, classLPC, classLFCC, classLineSpectralFrequencies, classMFCC, classVocalTractTier, nullptr);
-
+	Thing_recognizeClassesByName (classCepstrumc, classPowerCepstrum, classCepstrogram, classFormantPath, classFormantPathEditor, classPowerCepstrogram, classLPC, classLFCC, classLineSpectralFrequencies, classMFCC, classVocalTractTier, nullptr);
+	
+	structFormantPathEditor  :: f_preferences ();
+	
 	praat_addAction1 (classPowerCepstrum, 0, U"PowerCepstrum help", 0, 0, HELP_PowerCepstrum_help);
 	praat_addAction1 (classPowerCepstrum, 0, U"Draw...", 0, 0, GRAPHICS_PowerCepstrum_draw);
 	praat_addAction1 (classPowerCepstrum, 0, U"Draw trend line...", 0, 0, GRAPHICS_PowerCepstrum_drawTrendLine);
@@ -1057,15 +1255,29 @@ void praat_uvafon_LPC_init () {
 	praat_addAction1 (classFormant, 0, U"To LPC...", 0, 0, NEW_Formant_to_LPC);
 	praat_addAction1 (classFormant, 0, U"Formula...", U"Formula (bandwidths)...", 1, MODIFY_Formant_formula);
 	praat_addAction2 (classFormant, 1, classSpectrogram, 1, U"To IntensityTier...", 0, 0, NEW1_Formant_Spectrogram_to_IntensityTier);
+	
+	praat_addAction1 (classFormantPath, 1, U"View & Edit", 0, 0, WINDOW_FormantPath_viewAndEdit);
+	praat_addAction1 (classFormantPath, 0, U"Set navigation context...", 0, 0, MODIFY_FormantPath_setNavigationContext);
+	praat_addAction1 (classFormantPath, 0, U"Extract Formant", 0, 0, NEW_FormantPath_extractFormant);
+	praat_addAction1 (classFormantPath, 0, U"Extract TextGrid", 0, 0, NEW_FormantPath_extractTextGrid);
+	
+	praat_addAction2 (classFormantPath, 1, classStrings, 1, U"Set navigation labels...", 0, 0, MODIFY_FormantPath_setNavigationLabels);
+	praat_addAction2 (classFormantPath, 1, classStrings, 1, U"Set left context navigation labels...", 0, 0, MODIFY_FormantPath_setLeftContextNavigationLabels);
+	praat_addAction2 (classFormantPath, 1, classStrings, 1, U"Set right context navigation labels...", 0, 0, MODIFY_FormantPath_setRightContextNavigationLabels);
+	
 
-	
-	
 	praat_addAction1 (classLFCC, 0, U"LFCC help", 0, 0, HELP_LFCC_help);
 	praat_CC_init (classLFCC);
 	praat_addAction1 (classLFCC, 0, U"To LPC...", 0, 0, NEW_LFCC_to_LPC);
 
 	praat_addAction1 (classLineSpectralFrequencies, 0, U"LineSpectralFrequencies help", 0, 0, HELP_LineSpectralFrequencies_help);
 	praat_addAction1 (classLineSpectralFrequencies, 0, U"Draw frequencies...", 0, 0, GRAPHICS_LineSpectralFrequencies_drawFrequencies);
+	praat_addAction1 (classLineSpectralFrequencies, 0, QUERY_BUTTON, 0, 0, 0);
+		praat_TimeFrameSampled_query_init (classLineSpectralFrequencies);
+		praat_addAction1 (classLineSpectralFrequencies, 1, U"Get number of frequencies...", 0, 1, INTEGER_LineSpectralFrequencies_getNumberOfFrequencies);
+		praat_addAction1 (classLineSpectralFrequencies, 1, U"List frequencies in frame...", 0, 1, NUMVEC_LineSpectralFrequencies_listFrequenciesInFrame);
+		praat_addAction1 (classLineSpectralFrequencies, 1, U"List all frequencies", 0, 1, NUMMAT_LineSpectralFrequencies_listAllFrequencies);
+
 	praat_addAction1 (classLineSpectralFrequencies, 0, U"To LPC", 0, 0, NEW_LineSpectralFrequencies_to_LPC);
 	
 	praat_addAction1 (classLPC, 0, U"LPC help", 0, 0, HELP_LPC_help);
@@ -1075,7 +1287,13 @@ void praat_uvafon_LPC_init () {
 	praat_addAction1 (classLPC, 0, QUERY_BUTTON, 0, 0, 0);
 		praat_TimeFrameSampled_query_init (classLPC);
 		praat_addAction1 (classLPC, 1, U"Get sampling interval", 0, 1, REAL_LPC_getSamplingInterval);
+		praat_addAction1 (classLPC, 1, U"-- get coefficients --", nullptr, 1, nullptr);
 		praat_addAction1 (classLPC, 1, U"Get number of coefficients...", 0, 1, INTEGER_LPC_getNumberOfCoefficients);
+		praat_addAction1 (classLPC, 1, U"Get coefficients in frame...", 0, 1, NUMVEC_LPC_listAllCoefficientsInFrame);
+		praat_addAction1 (classLPC, 1, U"List all coefficients", 0, 1, NUMMAT_LPC_listAllCoefficients);
+		praat_addAction1 (classLPC, 1, U"-- get gain --", nullptr, 1, nullptr);
+		praat_addAction1 (classLPC, 1, U"Get gain in frame...", 0, 1, REAL_LPC_getGainInFrame);
+		praat_addAction1 (classLPC, 1, U"List all gains", 0, 1, NUMVEC_LPC_listAllGains);
 	praat_addAction1 (classLPC, 0, MODIFY_BUTTON, 0, 0, 0);
 		praat_TimeFunction_modify_init (classLPC);
 	praat_addAction1 (classLPC, 0, U"Extract", 0, 0, 0);
@@ -1093,7 +1311,7 @@ void praat_uvafon_LPC_init () {
 	praat_addAction1 (classLPC, 0, U"To LFCC...", 0, 0, NEW_LPC_to_LFCC);
 	praat_addAction1 (classLPC, 0, U"To Spectrogram...", 0, 0, NEW_LPC_to_Spectrogram);
 	praat_addAction1 (classLPC, 0, U"To LineSpectralFrequencies...", 0, 0, NEW_LPC_to_LineSpectralFrequencies);
-
+	
 	praat_addAction2 (classLPC, 1, classSound, 1, U"Analyse", 0, 0, 0);
 	praat_addAction2 (classLPC, 1, classSound, 1, U"Filter...", 0, 0, NEW1_LPC_Sound_filter);
 	praat_addAction2 (classLPC, 1, classSound, 1, U"Filter (inverse)", 0, 0, NEW1_LPC_Sound_filterInverse);
@@ -1101,15 +1319,18 @@ void praat_uvafon_LPC_init () {
 	praat_addAction2 (classLPC, 1, classSound, 1, U"Filter with filter at time...", 0, 0, NEW1_LPC_Sound_filterWithFilterAtTime);
 	praat_addAction2 (classLPC, 1, classSound, 1, U"Filter (inverse) with filter at time...", 0, 0, NEW1_LPC_Sound_filterInverseWithFilterAtTime);
 
-
-	praat_addAction1 (classSound, 0, U"To LPC (autocorrelation)...", U"To Formant (sl)...", 1, NEW_Sound_to_LPC_autocorrelation);
-	praat_addAction1 (classSound, 0, U"To LPC (covariance)...", U"To LPC (autocorrelation)...", 1, NEW_Sound_to_LPC_covariance);
-	praat_addAction1 (classSound, 0, U"To LPC (burg)...", U"To LPC (covariance)...", 1, NEW_Sound_to_LPC_burg);
-	praat_addAction1 (classSound, 0, U"To LPC (marple)...", U"To LPC (burg)...", 1, NEW_Sound_to_LPC_marple);
-	praat_addAction1 (classSound, 0, U"To MFCC...", U"To LPC (marple)...", 1, NEW_Sound_to_MFCC);
-	praat_addAction1 (classSound, 0, U"To Formant (robust)...", U"To Formant (sl)...", 2, NEW_Sound_to_Formant_robust);
 	praat_addAction1 (classSound, 0, U"To PowerCepstrogram...", U"To Harmonicity (gne)...", 1, NEW_Sound_to_PowerCepstrogram);
 	praat_addAction1 (classSound, 0, U"To PowerCepstrogram (hillenbrand)...", U"To Harmonicity (gne)...", praat_HIDDEN + praat_DEPTH_1, NEW_Sound_to_PowerCepstrogram_hillenbrand);
+	praat_addAction1 (classSound, 0, U"To Formant (robust)...", U"To Formant (sl)...", 2, NEW_Sound_to_Formant_robust);
+	praat_addAction1 (classSound, 0, U"To FormantPath...", U"To Formant (robust)...", 2, NEW_Sound_to_FormantPath);
+	praat_addAction1 (classSound, 0, U"To FormantPath (burg)...", U"To FormantPath...", 1, NEW_Sound_to_FormantPath_burg);
+	praat_addAction1 (classSound, 0, U"To LPC", U"To FormantPath...", 1, nullptr);
+	praat_addAction1 (classSound, 0, U"To LPC (autocorrelation)...", U"To LPC", 2, NEW_Sound_to_LPC_autocorrelation);
+	praat_addAction1 (classSound, 0, U"To LPC (covariance)...", U"To LPC (autocorrelation)...", 2, NEW_Sound_to_LPC_covariance);
+	praat_addAction1 (classSound, 0, U"To LPC (burg)...", U"To LPC (covariance)...", 2, NEW_Sound_to_LPC_burg);
+	praat_addAction1 (classSound, 0, U"To LPC (marple)...", U"To LPC (burg)...", 2, NEW_Sound_to_LPC_marple);
+	praat_addAction1 (classSound, 0, U"To MFCC...", U"To LPC (marple)...", 1, NEW_Sound_to_MFCC);
+	praat_addAction2 (classSound, 1, classTextGrid, 1, U"To FormantPath (burg)...", 0, 0, NEW_Sound_and_TextGrid_to_FormantPath_burg);
 	
 	praat_addAction1 (classVocalTract, 0, U"Draw segments...", U"Draw", 0, GRAPHICS_VocalTract_drawSegments);
 	praat_addAction1 (classVocalTract, 1, U"Get length", U"Draw segments...", 0, REAL_VocalTract_getLength);
