@@ -1,6 +1,6 @@
 /* NUM2.cpp
  *
- * Copyright (C) 1993-2019 David Weenink, Paul Boersma 2017
+ * Copyright (C) 1993-2020 David Weenink, Paul Boersma 2017,2020
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@
 
 #include "SVD.h"
 #include "Eigen.h"
-#include "NUMclapack.h"
+#include "NUMlapack.h"
 #include "NUM2.h"
 #include "NUMmachar.h"
 #include "melder.h"
@@ -116,10 +116,8 @@ void VECsmoothByMovingAverage_preallocated (VECVU const& out, constVECVU const&
 		integer jfrom = i - window / 2, jto = i + window / 2;
 		if (window % 2 == 0)
 			jto --;
-		if (jfrom < 1)
-			jfrom = 1;
-		if (jto > out.size)
-			jto = out.size;
+		Melder_clipLeft (1_integer, & jfrom);
+		Melder_clipRight (& jto, out.size);
 		out [i] = NUMmean (in.part (jfrom, jto));
 	}
 }
@@ -232,11 +230,10 @@ double NUMdeterminant_fromSymmetricMatrix (constMAT m) {
 	/*
 		Cholesky decomposition in lower, leave upper intact
 	*/
-	char uplo = 'U';
-	integer lda = m.nrow, info;
-	NUMlapack_dpotf2 (& uplo, & a.nrow, & a [1] [1], & lda, & info);
+	integer info;
+	NUMlapack_dpotf2_ ("U", a.nrow, & a [1] [1], m.nrow, & info);
 	Melder_require (info == 0,
-		U"dpotf2 cannot determine Cholesky decomposition.");
+			U"dpotf2_ failed with error = ", info);
 	longdouble lnd = 0.0;
 	for (integer i = 1; i <= a.nrow; i ++)
 		lnd += log (a [i] [i]);
@@ -262,39 +259,38 @@ autoMAT newMATlowerCholeslyInverse_fromLowerCholesky (constMAT const& m) {
 	return result;
 }
 
-void MATlowerCholesky_inplace (MAT a, double *out_lnd) {
+void MATlowerCholesky_inplace (MAT a, double *out_lnDeterminant) {
 	Melder_assert (a.nrow == a.ncol);
-	char uplo = 'U';
-	integer info;
 	/*
 		Cholesky decomposition in lower, leave upper intact
 		Fortran storage -> use uplo='U' to get 'L'.
 	*/
-	(void) NUMlapack_dpotf2 (& uplo, & a.nrow, & a [1] [1], & a.nrow, & info);
+	integer info;
+	NUMlapack_dpotf2_ ("U", a.nrow, & a [1] [1], a.nrow, & info);
 	Melder_require (info == 0,
 		U"dpotf2 fails with code ", info, U".");
 	/*
 		Determinant from diagonal, diagonal is now sqrt (a [i] [i]) !
 	*/
-	if (out_lnd) {
-		longdouble lnd = 0.0;
+	if (out_lnDeterminant) {
+		longdouble lnDeterminant = 0.0;
 		for (integer i = 1; i <= a.nrow; i ++)
-			lnd += log (a [i] [i]);
-		*out_lnd *= 2.0 * lnd; /* because A = L . L' */
+			lnDeterminant += log (a [i] [i]);
+		lnDeterminant *= 2.0;   /* because A = L . L' */
+		*out_lnDeterminant = double (lnDeterminant);
 	}
 }
 
 void MATlowerCholeskyInverse_inplace (MAT a, double *out_lnd) {
 	Melder_assert (a.nrow == a.ncol);
-	char uplo = 'U', diag = 'N';
 	MATlowerCholesky_inplace (a, out_lnd);
 	/*
 		Get the inverse
 	*/
 	integer info;
-	(void) NUMlapack_dtrtri (& uplo, & diag, & a.nrow, & a [1] [1], & a.nrow, & info);
+	(void) NUMlapack_dtrtri_ ("U", "N", a.nrow, & a [1] [1], a.nrow, & info);
 	Melder_require (info == 0,
-		U"dtrtri fails with code ", info, U".");
+		U"dtrtri_ fails with code ", info, U".");
 }
 
 autoMAT newMATinverse_fromLowerCholeskyInverse (constMAT m) {
@@ -406,9 +402,9 @@ void VECsolveNonnegativeLeastSquaresRegression (VECVU const& x, constMATVU const
 	autoVEC r = newVECraw (y.size);
 	const double normSquared_y = NUMsum2 (y);
 	integer iter = 1;
-	bool convergence = false;
+	bool farFromConvergence = true;
 	double difsq, difsq_previous = 1e100; // large enough
-	while (iter <= maximumNumberOfIterations && ! convergence) {
+	while (iter <= maximumNumberOfIterations && farFromConvergence) {
 		/*
 			Alternating Least Squares: Fixate all except x [icol]
 		*/
@@ -417,7 +413,7 @@ void VECsolveNonnegativeLeastSquaresRegression (VECVU const& x, constMATVU const
 			for (integer jcol = 1; jcol <= a.ncol; jcol ++)
 				if (jcol != icol)
 					r.get()  -=  x [jcol] * a.column (jcol);
-			const double ajr = NUMinner (a.column (icol), r);
+			const double ajr = NUMinner (a.column (icol), r.all());
 			const double ajaj = NUMsum2 (a.column (icol));
 			x [icol] = std::max (0.0, ajr / ajaj);
 		}
@@ -426,8 +422,8 @@ void VECsolveNonnegativeLeastSquaresRegression (VECVU const& x, constMATVU const
 		*/
 		VECmul (r.get(), a, x);
 		r.get()  -=  y;
-		difsq = NUMsum2 (r);
-		convergence = fabs (difsq - difsq_previous) < tol * normSquared_y;
+		difsq = NUMsum2 (r.all());
+		farFromConvergence = ( fabs (difsq - difsq_previous) > std::max (tol * normSquared_y, NUMeps) );
 		if (infoLevel > 1)
 			MelderInfo_writeLine (U"Iteration: ", iter, U", error: ", difsq);
 		difsq_previous = difsq;
@@ -560,8 +556,8 @@ void NUMsolveConstrainedLSQuadraticRegression (constMAT const& x, constVEC const
 			Case 3: page 634 use Newton-Raphson root finder
 		*/
 		struct nr_struct me;
-		me.y = y.at;
-		me.delta = delta.at;
+		me.y = y.asArgumentToFunctionThatExpectsOneBasedArray();
+		me.delta = delta.asArgumentToFunctionThatExpectsOneBasedArray();
 
 		double lambda = NUMnrbis (nr_func, delta [1] + eps, delta [2] - eps, & me);
 
@@ -710,7 +706,7 @@ autoVEC newVECsolveWeaklyConstrainedLinearRegression (constMAT const& a, constVE
 				x [j] /= c [j] - c [a.ncol]; // eq. 10
 			if (q > 1)
 				x.part (r + 2, a.ncol) <<= 0.0;
-			autoVEC result = newVECmul (u,  x);
+			autoVEC result = newVECmul (u, x.all());
 			return result;
 		}
 		// else continue with r = m - q
@@ -722,7 +718,7 @@ autoVEC newVECsolveWeaklyConstrainedLinearRegression (constMAT const& a, constVE
 	for (integer j = 1; j <= r; j ++)
 		xCx += x [j] * x [j] / c [j];
 
-	const double bmin = delta > 0.0 ? - xCx / delta : -2.0 * sqrt (alpha * xCx);
+	const double bmin = ( delta > 0.0 ? - xCx / delta : -2.0 * sqrt (alpha * xCx) );
 	const double eps = (c [a.ncol] - bmin) * tol;
 	/*
 		Find the root of d(psi(b)/db in interval (bmin, c [m])
@@ -735,7 +731,7 @@ autoVEC newVECsolveWeaklyConstrainedLinearRegression (constMAT const& a, constVE
 		x [j] /= c [j] - b0; // eq. 7
 	if (q > 1)
 		x.part (r + 1, a.ncol) <<= 0.0;
-	autoVEC result = newVECmul (u, x);
+	autoVEC result = newVECmul (u, x.all());
 	return result;
 }
 
@@ -1278,7 +1274,7 @@ void MATscaledResiduals (MAT const& residuals, constMAT const& data, constMAT co
 		autoMAT lowerInverse = newMATcopy (covariance);
 		MATlowerCholeskyInverse_inplace (lowerInverse.get(), nullptr);
 		for (integer irow = 1; irow <= data.nrow; irow ++) {
-			dif <<= data.row (irow)  -  means;
+			dif.all() <<= data.row (irow)  -  means;
 			residuals.row(irow) <<= 0.0;
 			if (lowerInverse.nrow == 1) { // diagonal matrix is one row matrix
 				residuals.row(irow) <<= lowerInverse.row(1)  *  dif.get();
@@ -1464,6 +1460,19 @@ autoVEC newVECburg (constVEC const& x, integer numberOfPredictionCoefficients, d
 	return a;
 }
 
+void VECfilterInverse_inplace (VEC const& s, constVEC const& filter, VEC const& filterMemory) {
+	Melder_assert (filterMemory.size >= filter.size);
+	filterMemory <<= 0.0;
+	for (integer i = 1; i <= s.size; i ++) {
+		const double y0 = s [i];
+		for (integer j = 1; j <= filter.size; j ++)
+			s [i] += filter [j] * filterMemory [j];
+		for (integer j = filter.size; j > 1; j --)
+			filterMemory [j] = filterMemory [j - 1];
+		filterMemory [1] = y0;
+	}
+}
+
 void NUMdmatrix_to_dBs (MAT const& m, double ref, double factor, double floor) {
 	const double factor10 = factor * 10.0;
 	MelderExtremaWithInit extrema;
@@ -1985,7 +1994,7 @@ void NUMlineFit_theil (constVEC const& x, constVEC const& y, double *out_m, doub
 		/*
 			Theil's incomplete method:
 			Split (x [i],y [i]) as
-			(x [i],y [i]), (x [N+i],y [N=i], i=1..numberOfPoints/2
+			(x [i],y [i]), (x [N+i],y [N+i], i=1..numberOfPoints/2
 			m [i] = (y [N+i]-y [i])/(x [N+i]-x [i])
 			m = median (m [i])
 			b = median(y [i]-m*x [i])
@@ -2002,7 +2011,7 @@ void NUMlineFit_theil (constVEC const& x, constVEC const& y, double *out_m, doub
 			autoVEC mbs;
 			if (! completeMethod) {
 				numberOfCombinations = x.size / 2;
-				mbs = newVECzero (x.size); // allocate twice to get the intercepts
+				mbs = newVECzero (x.size); // allocate for the intercept calculation too
 				integer n2 = x.size % 2 == 1 ? numberOfCombinations + 1 : numberOfCombinations;
 				for (integer i = 1; i <= numberOfCombinations; i ++)
 					mbs [i] = (y [n2 + i] - y [i]) / (x [n2 + i] - x [i]);
@@ -2129,7 +2138,6 @@ void VECarea_from_lpc (VEC area, constVEC lpc) {
 /*********** Begin deprecated LPC routines ***********************************/
 void NUMlpc_lpc_to_rc (double *lpc, integer p, double *rc) {
 	autoVEC b = newVECzero (p);
-	//autoNUMvector<double> a (NUMvector_copy<double> (lpc, 1, p), 1);
 	autoVEC a <<= VEC(lpc, p);
 	for (integer m = p; m > 0; m--) {
 		rc [m] = a [m];
@@ -2179,7 +2187,7 @@ void NUMlpc_area_to_lpc2 (double *area, integer n, double *lpc) {
 
 void NUMlpc_lpc_to_rc2 (double *lpc, integer m, double *rc);
 void NUMlpc_lpc_to_rc2 (double *lpc, integer m, double *rc) { // klopt nog niet
-	NUMvector_copyElements<double> (lpc, rc, 1, m);
+	rc.part(1,m) <<= lpc.part (1,m)
 	for (integer j = 2; j <= m; j ++) {
 		integer jb = m + 1 - j;
 		integer mh = (jb + 1) / 2;
@@ -2209,7 +2217,7 @@ void NUMlpc_area_to_rc (double *area, integer m, double *rc) {
 
 void NUMlpc_rc_to_lpc (double *rc, integer m, double *lpc);
 void NUMlpc_rc_to_lpc (double *rc, integer m, double *lpc) {
-	NUMvector_copyElements<double> (rc, lpc, 1, m);
+	lpc.part (1,m) <<= rc. part (1.m)
 	for (integer j = 2; j <= m; j ++) {
 		for (integer k = 1; k <= j / 2; k ++) {
 			double at = lpc [k] + rc [j] * lpc [j - k];
@@ -2838,20 +2846,20 @@ void MATmul3_XYsXt (MATVU const& target, constMAT const& x, constMAT const& y) {
 	3. Make all elements of v zero, except the numberOfNonZeros largest elements.
 	4. Set the support of these largest elements to 1 and the rest to zero.
 */
-static void VECupdateDataAndSupport_inplace (VECVU const& v, INTVECVU const& support, integer numberOfNonZeros) {
+static void VECupdateDataAndSupport_inplace (VECVU const& v, BOOLVECVU const& support, integer numberOfNonZeros) {
 	Melder_assert (v.size == support.size);
 	autoVEC abs = newVECabs (v);
 	autoINTVEC index = newINTVEClinear (v.size, 1, 1);
 	NUMsortTogether <double, integer> (abs.get(), index.get()); // sort is always increasing
 	for (integer i = 1; i <= v.size - numberOfNonZeros; i ++) {
 		v [index [i]] = 0.0;
-		support [index [i]] = 0;
+		support [index [i]] = false;
 	}
 	for (integer i = v.size - numberOfNonZeros + 1; i <= v.size; i ++)
-		support [index [i]] = 1;
+		support [index [i]] = true;
 }
 
-static double update (VEC const& x_new, VEC const& y_new, INTVEC const& support_new, constVECVU const& xn, double stepSize, constVEC const& gradient, constMATVU const& dictionary, constVEC const& yn, integer numberOfNonZeros, VEC buffer) {
+static double update (VEC const& x_new, VEC const& y_new, BOOLVECVU const& support_new, constVECVU const& xn, double stepSize, constVEC const& gradient, constMATVU const& dictionary, constVEC const& yn, integer numberOfNonZeros, VEC buffer) {
 	Melder_assert (x_new.size == xn.size && buffer.size == x_new.size);
 	Melder_assert (gradient.size == support_new.size && gradient.size == x_new.size);
 	Melder_assert (y_new.size == yn.size);
@@ -2881,6 +2889,13 @@ autoVEC newVECsolveSparse_IHT (constMATVU const& dictionary, constVECVU const& y
 	}
 }
 
+/* temporarily until present in melder_tensor.h */
+inline void operator<<= (BOOLVECVU const& target, constBOOLVECVU const& source) {
+	Melder_assert (target.size == source.size);
+	for (integer i = 1; i <= target.size; i ++)
+		target [i] = source [i];
+}
+
 void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECVU const& y, integer numberOfNonZeros, integer maximumNumberOfIterations, double tolerance, integer infoLevel) {
 	try {
 		Melder_assert (dictionary.ncol > dictionary.nrow); // must be underdetermined system
@@ -2893,8 +2908,8 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 		autoVEC yfromx_new = newVECraw (y.size); // D.x(n+1)
 		autoVEC ydif = newVECraw (y.size); // y - D.x(n)
 		autoVEC buffer = newVECraw (x.size);
-		autoINTVEC support = newINTVECraw (x.size);
-		autoINTVEC support_new = newINTVECraw (x.size);
+		autoBOOLVEC support = newBOOLVECraw (x.size);
+		autoBOOLVEC support_new = newBOOLVECraw (x.size);
 		
 		const double xnormSq = NUMsum2 (x);
 		const double rms_y = NUMsum2 (y) / y.size;
@@ -2908,15 +2923,15 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 			*/
 			VECmul (buffer.get(), dictionary.transpose(), y);
 			VECupdateDataAndSupport_inplace (buffer.get(), support.get(), numberOfNonZeros);
-			yfromx <<= 0.0;
-			ydif <<= y; // ydif = y - D.x(1) = y - D.0 = y
+			yfromx.all() <<= 0.0;
+			ydif.all() <<= y; // ydif = y - D.x(1) = y - D.0 = y
 		} else {
 			/*
 				We improve a current solution x
 			*/
 			VECupdateDataAndSupport_inplace (x, support.get(), numberOfNonZeros);
 			VECmul (yfromx.get(), dictionary, x); // D.x(n)
-			ydif <<= y  -  yfromx; // y - D.x(n)
+			ydif.all() <<= y  -  yfromx.all(); // y - D.x(n)
 			rms = NUMsum2 (ydif.get()) / y.size; // ||y - D.x(n)||^2
 		}
 		
@@ -2932,14 +2947,14 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 			*/
 			longdouble normsq_gs = 0.0; // squared norm of the sparse gradient
 			for (integer ig = 1; ig <= gradient.size; ig ++) {
-				if (support [ig] != 0)
+				if (support [ig])
 					normsq_gs += gradient [ig] * gradient [ig];
 			}
 			longdouble normsq_dgs = 0.0; // squared norm of the transformed sparse gradient
 			for (integer irow = 1; irow <= dictionary.nrow; irow ++) {
 				longdouble sum = 0.0;
 				for (integer icol = 1; icol <= dictionary.ncol; icol ++)
-					if (support [icol] != 0)
+					if (support [icol])
 						sum += dictionary [irow] [icol] * gradient [icol];
 				normsq_dgs += sum * sum;
 			}
@@ -2957,7 +2972,7 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 				}
 			}
 
-			ydif <<= y  -  yfromx_new; // y - D.x(n+1)
+			ydif.all() <<= y  -  yfromx_new.all();   // y - D.x(n+1)
 
 			const double rms_new = NUMsum2 (ydif.get()) / y.size;
 			const double relativeError = fabs (rms - rms_new) / rms_y;
@@ -2965,9 +2980,9 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 			if (infoLevel > 1)
 				MelderInfo_writeLine (U"Iteration: ", iter, U", error: ", rms_new, U" relative: ", relativeError, U" stepSize: ", stepSize);
 			
-			x <<= x_new;
-			support <<= support_new;
-			yfromx <<= yfromx_new;
+			x <<= x_new.all();
+			support.all() <<= support_new.all();
+			yfromx.all() <<= yfromx_new.all();
 			rms = rms_new;
 			iter ++;
 		}
@@ -2981,4 +2996,15 @@ void VECsolveSparse_IHT (VECVU const& x, constMATVU const& dictionary, constVECV
 	}
 }
 
+void NUMpolynomial_recurrence (VEC const& pn, double a, double b, double c, constVEC const& pnm1, constVEC const& pnm2) {
+	const integer degree = pn.size - 1;
+	Melder_assert (degree > 1 && pnm1.size >= pn.size && pnm2.size >= pn.size);
+
+	pn [1] = b * pnm1 [1] + c * pnm2 [1];
+	for (integer i = 2; i <= degree - 1; i ++)
+		pn [i] = a * pnm1 [i - 1] + b * pnm1 [i] + c * pnm2 [i];
+	pn [degree] = a * pnm1 [degree - 1] + b * pnm1 [degree];
+	pn [degree + 1] = a * pnm1 [degree];
+}
+
 /* End of file NUM2.cpp */
