@@ -1,6 +1,6 @@
 /* Configuration.cpp
  *
- * Copyright (C) 1993-2019 David Weenink
+ * Copyright (C) 1993-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
  djmw 20110304 Thing_new
  */
 
-#include <ctype.h>
 #include "SVD.h"
 #include "PCA.h"
 #include "Configuration.h"
@@ -254,7 +253,7 @@ static void NUMvarimax (MAT xm, MAT ym, bool normalizeRows, bool quartimax, inte
 		varianceSq_old = varianceSq;
 		varianceSq = NUMsquaredVariance (ym, quartimax);
 
-	} while (fabs (varianceSq_old - varianceSq) / varianceSq_old > tolerance &&
+	} while (fabs (varianceSq_old - varianceSq) > std::max (tolerance * varianceSq_old, NUMeps) &&
 	         numberOfIterations < maximumNumberOfIterations);
 
 	if (normalizeRows)
@@ -326,7 +325,7 @@ void Configuration_draw (Configuration me, Graphics g, int xCoordinate, int yCoo
 	}
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 	Graphics_setInner (g);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 	Graphics_setFontSize (g, labelSize);
 	for (integer i = 1; i <= my numberOfRows; i ++) {
 		if (x [i] >= xmin && x [i] <= xmax && y [i] >= ymin && y [i] <= ymax) {
