@@ -1,6 +1,6 @@
 /* MDS.cpp
  *
- * Copyright (C) 1993-2019 David Weenink, 2015,2017 Paul Boersma
+ * Copyright (C) 1993-2020 David Weenink, 2015,2017 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -259,7 +259,7 @@ autoMonotoneTransformator MonotoneTransformator_create (integer numberOfPoints)
 	try {
 		autoMonotoneTransformator me = Thing_new (MonotoneTransformator);
 		Transformator_init (me.get(), numberOfPoints);
-		my tiesHandling = kMDS_TiesHandling::PrimaryApproach;
+		my tiesHandling = kMDS_TiesHandling::PRIMARY_APPROACH;
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"MonotoneTransformator not created.");
@@ -316,7 +316,7 @@ autoDistance structISplineTransformator :: v_transform (MDSVec vec, Distance dis
 	for (integer iprox = 1; iprox <= numberOfProximities; iprox ++) {
 		const integer ii = vec->rowIndex [iprox];
 		const integer jj = vec->columnIndex [iprox];
-		const double r = NUMinner (m.row (iprox), b);
+		const double r = NUMinner (m.row (iprox), b.all());
 		thy data [ii] [jj] = thy data [jj] [ii] = r;
 	}
 	if (normalization)
@@ -608,7 +608,7 @@ void Salience_draw (Salience me, Graphics g, integer ix, integer iy, bool garnis
 	double ymax, xmax = ymax = wmax;
 	Graphics_setInner (g);
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 
 	for (integer i = 1; i <= my numberOfRows; i ++)
 		if (my rowLabels [i])
@@ -936,7 +936,7 @@ autoDistance MDSVec_Distance_monotoneRegression (MDSVec me, Distance thee, kMDS_
 		for (integer i = 1; i <= numberOfProximities; i ++)
 			distances [i] = thy data [my rowIndex [i]] [my columnIndex [i]];
 
-		if (tiesHandling == kMDS_TiesHandling::PrimaryApproach || tiesHandling == kMDS_TiesHandling::SecondaryApproach) {
+		if (tiesHandling == kMDS_TiesHandling::PRIMARY_APPROACH || tiesHandling == kMDS_TiesHandling::SECONDARY_APPROACH) {
 			/*
 				Kruskal's primary approach to tie-blocks:
 					Sort corresponding distances, with rowIndex, and columnIndex.
@@ -948,9 +948,9 @@ autoDistance MDSVec_Distance_monotoneRegression (MDSVec me, Distance thee, kMDS_
 				if (my proximity [i] == my proximity [i - 1])
 					continue;
 				if (i - ib > 1) {
-					if (tiesHandling == kMDS_TiesHandling::PrimaryApproach) {
+					if (tiesHandling == kMDS_TiesHandling::PRIMARY_APPROACH) {
 						// all equal
-					} else if (tiesHandling == kMDS_TiesHandling::SecondaryApproach) {
+					} else if (tiesHandling == kMDS_TiesHandling::SECONDARY_APPROACH) {
 						const double mean = NUMmean (distances.part (ib, i - 1));
 						distances.part (ib, i - 1) <<= mean;
 					}
@@ -1205,18 +1205,18 @@ double Distance_Weight_stress (Distance fit, Distance conf, Weight weight, kMDS_
 		scale of the configuration, i.e., the distances conf [i] [j].
 	*/
 
-	if (stressMeasure == kMDS_stressMeasure::Normalized) {
+	if (stressMeasure == kMDS_stressMeasure::NORMALIZED) {
 		const double denum = eta_fit * eta_conf;
 		if (denum > 0.0)
 			stress = 1.0 - rho * rho / denum;
-	} else if (stressMeasure == kMDS_stressMeasure::Kruskal_1) {
+	} else if (stressMeasure == kMDS_stressMeasure::KRUSKAL_1) {
 		const double denum = eta_fit * eta_conf;
 		if (denum > 0.0) {
 			const double tmp = 1.0 - rho * rho / denum;
 			if (tmp > 0.0)
 				stress = sqrt (tmp);
 		}
-	} else if (stressMeasure == kMDS_stressMeasure::Kruskal_2) {
+	} else if (stressMeasure == kMDS_stressMeasure::KRUSKAL_2) {
 		/*
 			Get average distance.
 		*/
@@ -1243,7 +1243,7 @@ double Distance_Weight_stress (Distance fit, Distance conf, Weight weight, kMDS_
 			if (denum > 0.0)
 				stress = sqrt ((eta_fit * eta_conf - rho * rho) / denum);
 		}
-	} else if (stressMeasure == kMDS_stressMeasure::Raw) {
+	} else if (stressMeasure == kMDS_stressMeasure::RAW) {
 		stress = eta_fit + eta_conf - 2.0 * rho;
 	}
 	return stress;
@@ -1405,7 +1405,7 @@ autoConfiguration Dissimilarity_Configuration_Weight_Transformator_smacof (Dissi
 			*/
 			autoDistance cdist = Configuration_to_Distance (conf);
 
-			stress = Distance_Weight_stress (fit.get(), cdist.get(), weight, kMDS_stressMeasure::Normalized);
+			stress = Distance_Weight_stress (fit.get(), cdist.get(), weight, kMDS_stressMeasure::NORMALIZED);
 			/*
 				Check stop criterium
 			*/
@@ -1517,7 +1517,7 @@ autoConfiguration Dissimilarity_Configuration_Weight_ispline_mds (Dissimilarity
 
 autoConfiguration Dissimilarity_Weight_absolute_mds (Dissimilarity me, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress) {
 	try {
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Absolute);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::ABSOLUTE_);
 		autoConfiguration cstart = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		autoConfiguration c = Dissimilarity_Configuration_Weight_absolute_mds (me, cstart.get(), w, tolerance, numberOfIterations, numberOfRepetitions, showProgress);
 		return c;
@@ -1528,7 +1528,7 @@ autoConfiguration Dissimilarity_Weight_absolute_mds (Dissimilarity me, Weight w,
 
 autoConfiguration Dissimilarity_Weight_interval_mds (Dissimilarity me, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress) {
 	try {
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ratio);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::RATIO);
 		autoConfiguration cstart = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		autoConfiguration c = Dissimilarity_Configuration_Weight_interval_mds (me, cstart.get(), w, tolerance, numberOfIterations, numberOfRepetitions, showProgress);
 		return c;
@@ -1539,7 +1539,7 @@ autoConfiguration Dissimilarity_Weight_interval_mds (Dissimilarity me, Weight w,
 
 autoConfiguration Dissimilarity_Weight_monotone_mds (Dissimilarity me, Weight w, integer numberOfDimensions, kMDS_TiesHandling tiesHandling, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress) {
 	try {
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ordinal);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::ORDINAL);
 		autoConfiguration cstart = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		autoConfiguration c = Dissimilarity_Configuration_Weight_monotone_mds (me, cstart.get(), w, tiesHandling, tolerance, numberOfIterations, numberOfRepetitions, showProgress);
 		return c;
@@ -1550,7 +1550,7 @@ autoConfiguration Dissimilarity_Weight_monotone_mds (Dissimilarity me, Weight w,
 
 autoConfiguration Dissimilarity_Weight_ratio_mds (Dissimilarity me, Weight w, integer numberOfDimensions, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress) {
 	try {
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ratio);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::RATIO);
 		autoConfiguration cstart = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		autoConfiguration c = Dissimilarity_Configuration_Weight_ratio_mds (me, cstart.get(), w, tolerance,
 		    numberOfIterations, numberOfRepetitions, showProgress);
@@ -1562,7 +1562,7 @@ autoConfiguration Dissimilarity_Weight_ratio_mds (Dissimilarity me, Weight w, in
 
 autoConfiguration Dissimilarity_Weight_ispline_mds (Dissimilarity me, Weight w, integer numberOfDimensions, integer numberOfInteriorKnots, integer order, double tolerance, integer numberOfIterations, integer numberOfRepetitions, bool showProgress) {
 	try {
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ordinal);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::ORDINAL);
 		autoConfiguration cstart = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		autoConfiguration c = Dissimilarity_Configuration_Weight_ispline_mds (me, cstart.get(), w,
 		    numberOfInteriorKnots, order, tolerance, numberOfIterations, numberOfRepetitions, showProgress);
@@ -1578,7 +1578,7 @@ static void MDSVec_Distances_getStressValues (MDSVec me, Distance ddist, Distanc
 	
 	longdouble s = 0.0, t = 0.0, dbar = 0.0;
 
-	if (stress_formula == kMDS_KruskalStress::Kruskal_1) {
+	if (stress_formula == kMDS_KruskalStress::KRUSKAL_1) {
 		for (integer i = 1; i <= my numberOfProximities; i ++) {
 			dbar += ddist -> data [my rowIndex [i]] [my columnIndex [i]];
 		}
@@ -1609,8 +1609,8 @@ autoKruskal Kruskal_create (integer numberOfPoints, integer numberOfDimensions)
 		my configuration = Configuration_create (numberOfPoints, numberOfDimensions);
 		my proximities = ProximityList_create ();
 		my dx = newMATraw (numberOfPoints, numberOfDimensions);
-		my tiesHandling = kMDS_TiesHandling::PrimaryApproach;
-		my stress_formula = kMDS_KruskalStress::Kruskal_1;
+		my tiesHandling = kMDS_TiesHandling::PRIMARY_APPROACH;
+		my stress_formula = kMDS_KruskalStress::KRUSKAL_1;
 		return me;
 	} catch (MelderError) {
 		Melder_throw (U"Kruskal not created.");
@@ -1619,8 +1619,8 @@ autoKruskal Kruskal_create (integer numberOfPoints, integer numberOfDimensions)
 
 autoConfiguration Dissimilarity_to_Configuration_kruskal (Dissimilarity me, integer numberOfDimensions, integer /* metric */, kMDS_TiesHandling tiesHandling, kMDS_KruskalStress stress_formula, double tolerance, integer numberOfIterations, integer numberOfRepetitions) {
 	try {
-		//autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ratio);
-		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::Ordinal);
+		//autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::RATIO);
+		autoDistance d = Dissimilarity_to_Distance (me, kMDS_AnalysisScale::ORDINAL);
 		autoConfiguration c = Distance_to_Configuration_torsca (d.get(), numberOfDimensions);
 		Configuration_normalize (c.get(), 1.0, false);
 		autoConfiguration thee = Dissimilarity_Configuration_kruskal (me, c.get(), tiesHandling, stress_formula, tolerance, numberOfIterations, numberOfRepetitions);
@@ -1727,7 +1727,7 @@ double Dissimilarity_Configuration_Weight_Transformator_normalizedStress (Dissim
 	autoDistance cdist = Configuration_to_Distance (conf);
 	autoMDSVec vec = Dissimilarity_to_MDSVec (me);
 	autoDistance fdist = Transformator_transform (t, vec.get(), cdist.get(), weight);
-	const double stress = Distance_Weight_stress (fdist.get(), cdist.get(), weight, kMDS_stressMeasure::Normalized);
+	const double stress = Distance_Weight_stress (fdist.get(), cdist.get(), weight, kMDS_stressMeasure::NORMALIZED);
 	return stress;
 }
 
@@ -2118,7 +2118,7 @@ void DissimilarityList_indscal (DissimilarityList me, integer numberOfDimensions
 	try {
 		const bool showSingle = (showProgress && numberOfRepetitions == 1);
 
-		autoDistanceList distances = DissimilarityList_to_DistanceList (me, kMDS_AnalysisScale::Ordinal);
+		autoDistanceList distances = DissimilarityList_to_DistanceList (me, kMDS_AnalysisScale::ORDINAL);
 		autoConfiguration cstart; autoSalience wstart;
 		DistanceList_to_Configuration_ytl (distances.get(), numberOfDimensions, normalizeScalarProducts, & cstart, & wstart);
 		autoConfiguration conf = Data_copy (cstart.get());
@@ -2338,7 +2338,7 @@ void drawSplines (Graphics g, double low, double high, double ymin, double ymax,
 	double knot [maximumNumberOfKnots + 1];
 	if (k > maximumNumberOfKnots)
 		return;
-	if (splineType == kMDS_splineType::ISpline)
+	if (splineType == kMDS_splineType::I_SPLINE)
 		k ++;
 	for (integer i = 1; i <= k; i ++)
 		knot [i] = low;
@@ -2384,20 +2384,20 @@ void drawSplines (Graphics g, double low, double high, double ymin, double ymax,
 		for (integer j = 1; j <= numberOfPoints; j ++) {
 			const double x = low + dx * (j - 1);
 			double yx;
-			if (splineType == kMDS_splineType::MSpline)
-				yx = NUMmspline (constVEC (knot, numberOfKnots), order, i, x);
+			if (splineType == kMDS_splineType::M_SPLINE)
+				yx = NUMmspline (constVEC (& knot [1], numberOfKnots), order, i, x);
 			else
-				yx = NUMispline (constVEC (knot, numberOfKnots), order, i, x);
+				yx = NUMispline (constVEC (& knot [1], numberOfKnots), order, i, x);
 			y [j] = ( yx < ymin ? ymin : ( yx > ymax ? ymax : yx ) );
 		}
-		Graphics_function (g, y.at, 1, numberOfPoints, low, high);
+		Graphics_function (g, y.asArgumentToFunctionThatExpectsOneBasedArray(), 1, numberOfPoints, low, high);
 	}
 	Graphics_unsetInner (g);
 	if (garnish) {
-		static MelderString ts { };
-		const integer lastKnot = ( splineType == kMDS_splineType::ISpline ? numberOfKnots - 2 : numberOfKnots );
+		static MelderString ts;
+		const integer lastKnot = ( splineType == kMDS_splineType::I_SPLINE ? numberOfKnots - 2 : numberOfKnots );
 		Graphics_drawInnerBox (g);
-		Graphics_textLeft (g, false, ( splineType == kMDS_splineType::MSpline ? U"\\s{M}\\--spline" : U"\\s{I}\\--spline" ));
+		Graphics_textLeft (g, false, ( splineType == kMDS_splineType::M_SPLINE ? U"\\s{M}\\--spline" : U"\\s{I}\\--spline" ));
 		Graphics_marksTop (g, 2, true, true, false);
 		Graphics_marksLeft (g, 2, true, true, false);
 		if (low <= knot [order]) {
@@ -2439,7 +2439,7 @@ void drawMDSClassRelations (Graphics g) {
 	const conststring32 text [7] = {U"", U"Confusion", U"Dissimilarity  %\\de__%%ij%_",  U"Similarity", U"Distance  %d__%%ij%_, %d\\'p__%%ij%_", U"ScalarProduct", U"Configuration" };
 
 	Graphics_setWindow (g, -0.05, 1.05, -0.05, 1.05);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 	for (integer i = 1; i <= nBoxes; i ++) {
 		x2 = x [i] + boxWidth;
 		y2 = y [i] + boxHeight;
@@ -2527,9 +2527,9 @@ void drawMDSClassRelations (Graphics g) {
 	Graphics_line (g, x13, y1, x23, y1);
 	Graphics_arrow (g, x23, y1, x23, y [6] + boxHeight);
 	x1 = x [6] + boxWidth + dboxx / 2;
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_BOTTOM);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_BOTTOM);
 	Graphics_text (g, x1, y1, U"\\s{TORSCA}");
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_TOP);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_TOP);
 	Graphics_text (g, x1, y1, U"\\s{YTL}");
 
 	Graphics_setLineType (g, Graphics_DOTTED);
@@ -2539,7 +2539,7 @@ void drawMDSClassRelations (Graphics g) {
 	Graphics_line (g, x23, y [5], x23, ym);
 	Graphics_arrow (g, x23, ym, x [6] + boxWidth, ym);
 	x1 = x [6] + boxWidth + dboxx / 2 + boxWidth3;
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_BOTTOM);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_BOTTOM);
 	Graphics_text (g, x1, ym, U"\\s{INDSCAL}");
 
 	// Dissimilarity to Configuration
