@@ -1,6 +1,6 @@
 /* Matrix_extensions.cpp
  *
- * Copyright (C) 1993-2019 David Weenink
+ * Copyright (C) 1993-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,17 +25,11 @@
  djmw 20080122 float -> double
 */
 
-#include "Graphics_extensions.h"
 #include "Matrix_extensions.h"
 #include "Eigen.h"
 #include "NUM2.h"
 #include "Permutation.h"
 
-#include "enums_getText.h"
-#include "Matrix_extensions_enums.h"
-#include "enums_getValue.h"
-#include "Graphics_extensions_enums.h"
-
 void Matrix_scatterPlot (Matrix me, Graphics g, integer icx, integer icy,
 	double xmin, double xmax, double ymin, double ymax,
 	double size_mm, conststring32 mark, bool garnish)
@@ -97,16 +91,16 @@ void Matrix_drawAsSquares_inside (Matrix me, Graphics g, double xmin, double xma
 	const integer numberOfCells = numberOfRows * numberOfColumns;
 	autoPermutation p = Permutation_create (numberOfCells);
 	
-	if (drawingOrder == kGraphicsMatrixCellDrawingOrder::Rows) {
+	if (drawingOrder == kGraphicsMatrixCellDrawingOrder::ROWS) {
 		// identity permutation
-	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::Random) {
+	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::RANDOM) {
 		Permutation_permuteRandomly_inplace (p.get(), 1, numberOfCells);
-	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::IncreasingValues || drawingOrder == kGraphicsMatrixCellDrawingOrder::DecreasingValues) {
+	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::INCREASING_VALUES || drawingOrder == kGraphicsMatrixCellDrawingOrder::DECREASING_VALUES) {
 		autoVEC v = nummat_vectorize (my z.get(), rowmin, rowmax, colmin, colmax);
 		NUMsortTogether (v.get(), p -> p.get());
-		if (drawingOrder == kGraphicsMatrixCellDrawingOrder::DecreasingValues)
+		if (drawingOrder == kGraphicsMatrixCellDrawingOrder::DECREASING_VALUES)
 			Permutation_reverse_inline (p.get(), 1, numberOfCells);
-	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::Columns) {
+	} else if (drawingOrder == kGraphicsMatrixCellDrawingOrder::COLUMNS) {
 		Permutation_tableJump_inline (p.get(), numberOfColumns, 1);
 	}
 	
@@ -122,13 +116,13 @@ void Matrix_drawAsSquares_inside (Matrix me, Graphics g, double xmin, double xma
 		const double halfCellWidth = xfraction * 0.5 * my dx * scaleFactor;
 		const double halfCellHeight = yfraction * 0.5 * my dy * scaleFactor;
 		double cellLeft, cellTop;
-		if (origin == kGraphicsMatrixOrigin::TopLeft) {
+		if (origin == kGraphicsMatrixOrigin::TOP_LEFT) {
 			cellLeft = Matrix_columnToX (me, icol) - halfCellWidth;
 			cellTop = Matrix_rowToY (me, rowmax - irow + rowmin) + halfCellHeight;
-		} else if (origin == kGraphicsMatrixOrigin::TopRight) {
+		} else if (origin == kGraphicsMatrixOrigin::TOP_RIGHT) {
 			cellLeft = Matrix_columnToX (me, colmax - icol + colmin) - halfCellWidth;
 			cellTop = Matrix_rowToY (me, rowmax - irow + rowmin) + halfCellHeight;
-		} else if (origin == kGraphicsMatrixOrigin::BottomLeft) {
+		} else if (origin == kGraphicsMatrixOrigin::BOTTOM_LEFT) {
 			cellLeft = Matrix_columnToX (me, icol) - halfCellWidth;
 			cellTop = Matrix_rowToY (me, irow) + halfCellHeight;
 		} else { // origin == kGraphicsMatrixOrigin::BottomRight
@@ -158,7 +152,7 @@ void Matrix_drawAsSquares (Matrix me, Graphics g, double xmin, double xmax, doub
 	Graphics_setInner (g);
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 	
-	Matrix_drawAsSquares_inside (me, g, xmin, xmax, ymin, ymax, kGraphicsMatrixOrigin::BottomLeft, 0.95 * 0.95, kGraphicsMatrixCellDrawingOrder::Rows);
+	Matrix_drawAsSquares_inside (me, g, xmin, xmax, ymin, ymax, kGraphicsMatrixOrigin::BOTTOM_LEFT, 0.95 * 0.95, kGraphicsMatrixCellDrawingOrder::ROWS);
 	
 	Graphics_setGrey (g, 0.0);
 	Graphics_unsetInner (g);
@@ -310,14 +304,15 @@ void Matrix_drawSliceY (Matrix me, Graphics g, double x, double ymin, double yma
 		min -= 0.5;
 		max += 0.5;
 	}
-	autoNUMvector<double> y (iymin, iymax);
+	const integer ysize = iymax - iymin + 1;
+	autoVEC y = newVECraw (ysize);
 
 	Graphics_setWindow (g, ymin, ymax, min, max);
 	Graphics_setInner (g);
 
 	for (integer i = iymin; i <= iymax; i ++)
-		y [i] = my z [i] [ix];
-	Graphics_function (g, y.peek(), iymin, iymax, Matrix_rowToY (me, iymin), Matrix_rowToY (me, iymax));
+		y [i - iymin + 1] = my z [i] [ix];
+	Graphics_function (g, y.asArgumentToFunctionThatExpectsOneBasedArray(), 1, ysize, Matrix_rowToY (me, iymin), Matrix_rowToY (me, iymax));
 	Graphics_unsetInner (g);
 }
 
@@ -338,7 +333,7 @@ autoMatrix Matrix_solveEquation (Matrix me, double tolerance) {
 		b.get() <<= my z.column (my nx);
 
 		autoVEC x = newVECsolve (u.get(), b.get(), tolerance);
-		thy z.row (1) <<= x;
+		thy z.row (1) <<= x.all();
 		
 		return thee;
 	} catch (MelderError) {
@@ -559,29 +554,63 @@ void Matrix_Eigen_complex (Matrix me, autoMatrix *out_eigenvectors, autoMatrix *
 	try {
 		Melder_require (my nx == my ny,
 			U"The Matrix should be square.");
-		autoVEC eigenvalues_re, eigenvalues_im;
-		autoMAT right_eigenvectors;
-		MAT_getEigenSystemFromGeneralMatrix (my z.get(), nullptr, & right_eigenvectors, & eigenvalues_re, & eigenvalues_im);
-		autoMAT eigenvectors_reim;
-		MAT_eigenvectors_decompress (right_eigenvectors.get(), eigenvalues_re.get(), eigenvalues_im.get(), & eigenvectors_reim);
+		Melder_require ((out_eigenvectors || out_eigenvalues),
+			U"You should want either eigenvalues or eigenvectors or both to be calculated.");
+		
+		autoCOMPVEC eigenvalues;
+		autoCOMPVEC *p_eigenvalues = ( out_eigenvalues ? & eigenvalues : nullptr );
+		automatrix<dcomplex> eigenvectors;
+		automatrix<dcomplex> *p_eigenvectors = ( out_eigenvectors ? & eigenvectors : nullptr );
+		
+		MAT_getEigenSystemFromGeneralSquareMatrix (my z.get(), p_eigenvalues, p_eigenvectors);
+	
 		if (out_eigenvectors) {
-			autoMatrix eigenvectors = Matrix_createSimple (my ny, 2 * my ny);
-			eigenvectors -> z.all() <<= eigenvectors_reim.all();
-			*out_eigenvectors = eigenvectors.move();
+			autoMatrix eigenvectorsM = Matrix_createSimple (my ny, 2 * my ny);
+			for (integer ivec = 1; ivec <= eigenvectors.ncol; ivec ++)
+				for (long irow = 1; irow <= my ny; irow ++) {
+					eigenvectorsM -> z [irow] [2 * ivec - 1] = eigenvectors [irow] [ivec] .real();
+					eigenvectorsM -> z [irow] [2 * ivec    ] = eigenvectors [irow] [ivec] .imag();
+				}
+			*out_eigenvectors = eigenvectorsM.move();
 		}
 		if (out_eigenvalues) {
-			autoMatrix eigenvalues = Matrix_createSimple (my ny, 2);
+			autoMatrix eigenvaluesM = Matrix_createSimple (my ny, 2);
 			for (long i = 1; i <= my ny; i ++) {
-				eigenvalues -> z [i] [1] = eigenvalues_re [i];
-				eigenvalues -> z [i] [2] = eigenvalues_im [i];
+				eigenvaluesM -> z [i] [1] = eigenvalues [i] .real();
+				eigenvaluesM -> z [i] [2] = eigenvalues [i] .imag();
 			}
-			*out_eigenvalues = eigenvalues.move();	
+			*out_eigenvalues = eigenvaluesM.move();	
 		}
 	} catch (MelderError) {
 		Melder_throw (U"Cannot create Eigenvalues from Matrix.");
 	}
 }
 
+autoCOMPVEC Matrix_listEigenvalues (Matrix me) {
+	Melder_require (my nx == my ny,
+		U"The Matrix should be square.");
+	autoCOMPVEC eigenvalues;
+	MAT_getEigenSystemFromGeneralSquareMatrix (my z.get(), & eigenvalues, nullptr);
+	return eigenvalues;
+}
+
+automatrix<dcomplex> Matrix_listEigenvectors (Matrix me) {
+	Melder_require (my nx == my ny,
+		U"The Matrix should be square.");
+	automatrix<dcomplex> eigenvectors;
+	
+	MAT_getEigenSystemFromGeneralSquareMatrix (my z.get(), nullptr,& eigenvectors);
+	integer numberOfEigenvectors = eigenvectors.nrow;
+	automatrix<dcomplex> result = newmatrixraw<dcomplex> (eigenvectors.ncol, eigenvectors.nrow);
+	/*
+		vec's vertical
+	*/
+	for (integer ivec = 1; ivec <= numberOfEigenvectors; ivec ++)
+		for (long irow = 1; irow <= result.nrow; irow ++)
+			result [irow] [ivec] = eigenvectors [ivec] [irow];
+	return result;
+}
+
 autoMatrix SVD_to_Matrix (SVD me, integer from, integer to) {
 	try {
 		autoMAT synthesis = SVD_synthesize (me, from, to);
