@@ -1,6 +1,6 @@
 /* Minimizers.cpp
  *
- * Copyright (C) 2001-2019 David Weenink
+ * Copyright (C) 2001-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ void Minimizer_minimize (Minimizer me, integer maximumNumberOfIterations, double
 
 		if (my iteration + maximumNumberOfIterations > my maximumNumberOfIterations) {
 			my maximumNumberOfIterations += maximumNumberOfIterations;
-			my history.resize (my maximumNumberOfIterations);
+			my history. resize (my maximumNumberOfIterations);
 		}
 		if (monitor)
 			my gmonitor = (Graphics) Melder_monitor (0.0, U"Starting...");
@@ -127,7 +127,7 @@ void Minimizer_reset (Minimizer me, constVEC const& guess) {
 		for (integer i = 1; i <= my numberOfParameters; i ++)
 			my p [i] = NUMrandomUniform (-1.0, 1.0);
 
-	my history.resize (0);
+	my history. resize (0);
 	my maximumNumberOfIterations = my numberOfFunctionCalls = my iteration = 0;
 	my success = false;
 	my minimum = 1.0e38;
@@ -156,7 +156,7 @@ void Minimizer_drawHistory (Minimizer me, Graphics g, integer iFrom, integer iTo
 	}
 	Graphics_setInner (g);
 	Graphics_setWindow (g, iFrom, iTo, hmin, hmax);
-	Graphics_function (g, my history.at, itmin, itmax, itmin, itmax);
+	Graphics_function (g, my history.asArgumentToFunctionThatExpectsOneBasedArray(), itmin, itmax, itmin, itmax);
 	Graphics_unsetInner (g);
 	if (garnish) {
 		Graphics_drawInnerBox (g);
@@ -342,9 +342,9 @@ void structVDSmagtMinimizer :: v_minimize () {
 				if ((fc < minimum) || ((fc == minimum) && (grc / gropt > -1))) {
 					gopt_sq = gsq;
 					history [this -> iteration] = minimum = fc;
-					std::swap (p.at, pc.at);
-					std::swap (dp.at, gc.at);
-					if (grc * gropt <= 0)
+					std::swap (p, pc);
+					std::swap (dp, gc);
+					if (grc * gropt <= 0.0)
 						alplim = alphamin;
 					alphamin = alpha;
 					gropt = grc;
