@@ -1,6 +1,6 @@
 /* TableOfReal_extensions.cpp
  *
- * Copyright (C) 1993-2019 David Weenink, 2017 Paul Boersma
+ * Copyright (C) 1993-2020 David Weenink, 2017 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,11 +48,10 @@
  djmw 20111123 Always use Melder_wcscmp
 */
 
-#include <ctype.h>
+#include "Covariance.h"
 #include "Graphics_extensions.h"
-#include "SSCP.h"
 #include "Matrix_extensions.h"
-#include "NUMclapack.h"
+#include "NUMlapack.h"
 #include "NUM2.h"
 #include "SVD.h"
 #include "Table_extensions.h"
@@ -271,7 +270,7 @@ void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, conststring32
 	Melder_require (colb > 0 && colb <= cole && cole <= my numberOfColumns,
 		U"Invalid columns");
 
-	autoVEC irows = VEC_createFromString (rows);
+	autoVEC irows = newVECfromString (rows);
 	for (integer i = 1; i <= irows.size; i ++) {
 		const integer irow = Melder_ifloor (irows [i]);
 		Melder_require (irow > 0 && irow <= my numberOfRows,
@@ -291,7 +290,7 @@ void TableOfReal_drawRowsAsHistogram (TableOfReal me, Graphics g, conststring32
 		}
 	}
 
-	autoVEC igreys = VEC_createFromString (greys);
+	autoVEC igreys = newVECfromString (greys);
 
 	Graphics_setWindow (g, 0.0, 1.0, ymin, ymax);
 	Graphics_setInner (g);
@@ -382,7 +381,7 @@ void TableOfReal_drawBiplot (TableOfReal me, Graphics g, double xmin, double xma
 	Graphics_setInner (g);
 	if (labelsize > 0)
 		Graphics_setFontSize (g, labelsize);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 
 	for (integer i = 1; i <= nPoints; i ++) {
 		char32 const *label;
@@ -652,7 +651,7 @@ void TableOfReal_drawScatterPlotMatrix (TableOfReal me, Graphics g, integer colb
 	Graphics_setInner (g);
 	Graphics_line (g, 0.0, numberOfColumns, numberOfColumns, numberOfColumns);
 	Graphics_line (g, 0.0, 0.0, 0.0, numberOfColumns);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 
 	for (integer icol = 1; icol <= numberOfColumns; icol ++) {
 		const integer ycol = colb + icol - 1;
@@ -731,7 +730,7 @@ void TableOfReal_drawScatterPlot (TableOfReal me, Graphics g, integer icx, integ
 	}
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 	Graphics_setInner (g);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 	Graphics_setFontSize (g, labelSize);
 
 	for (integer i = rowb; i <= rowe; i ++) {
@@ -1035,7 +1034,7 @@ void TableOfReal_drawVectors (TableOfReal me, Graphics g, integer colx1, integer
 
 	Graphics_setWindow (g, xmin, xmax, ymin, ymax);
 	Graphics_setInner (g);
-	Graphics_setTextAlignment (g, Graphics_CENTRE, Graphics_HALF);
+	Graphics_setTextAlignment (g, kGraphics_horizontalAlignment::CENTRE, Graphics_HALF);
 
 	if (labelsize > 0)
 		Graphics_setFontSize (g, labelsize);
@@ -1231,7 +1230,6 @@ autoTableOfReal TableOfReal_to_TableOfReal (TableOfReal me) {
 
 autoTableOfReal TableOfReal_choleskyDecomposition (TableOfReal me, bool upper, bool inverse) {
 	try {
-		char diag = 'N';
 		integer n = my numberOfColumns, lda = my numberOfRows, info;
 
 		Melder_require (n == lda,
@@ -1248,13 +1246,13 @@ autoTableOfReal TableOfReal_choleskyDecomposition (TableOfReal me, bool upper, b
 				for (integer j = i + 1; j <= n; j ++)
 					thy data [i] [j] = 0.0;
 		}
-		char uplo = upper ? 'L' : 'U';
-		NUMlapack_dpotf2 (& uplo, & n, & thy data [1] [1], & lda, & info);
+		const char *uplo = ( upper ? "L" : "U" );
+		NUMlapack_dpotf2_ (uplo, n, & thy data [1] [1], lda, & info);
 		Melder_require (info == 0,
 			U"dpotf2 fails");
 		
 		if (inverse) {
-			NUMlapack_dtrtri (&uplo, &diag, &n, &thy data [1] [1], &lda, &info);
+			NUMlapack_dtrtri_ (uplo, "N", n, & thy data [1] [1], lda, &info);
 			Melder_require (info == 0,
 				U"dtrtri fails");
 		}
