@@ -1,6 +1,6 @@
 /* praat_DataModeler_init.cpp
  *
- * Copyright (C) 2014-2017 David Weenink
+ * Copyright (C) 2014-2020 David Weenink
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
 #include "praatP.h"
 #include "DataModeler.h"
 #include "Formant_extensions.h"
+#include "FormantModeler.h"
 #include "OptimalCeilingTierEditor.h"
 #include "Pitch.h"
 #include "Table_extensions.h"
-#include <math.h>
+#include "TextGrid.h"
 
 #undef iam
 #define iam iam_LOOP
@@ -36,14 +37,11 @@ FORM (NEW1_DataModeler_createSimple, U"Create simple DataModeler", nullptr) {
 	NATURAL (numberOfDataPoints, U"Number of data points", U"20")
 	SENTENCE (parameter_string, U"Parameters", U"0.0 1.0 1.0")
 	POSITIVE (standardDeviation, U"Gaussian noise stdev", U"0.2")
-	OPTIONMENU (functionType, U"Basis functions", 2)
-		OPTION (U"Polynomial")
-		OPTION (U"Legendre")
-		
+	OPTIONMENU_ENUM (kDataModelerFunction, type, U"Basis functions", kDataModelerFunction::DEFAULT)		
 	OK
 DO
 	CREATE_ONE
-		autoDataModeler result = DataModeler_createSimple (xmin, xmax, numberOfDataPoints, parameter_string, standardDeviation, functionType - 1);
+		autoDataModeler result = DataModeler_createSimple (xmin, xmax, numberOfDataPoints, parameter_string, standardDeviation, type);
 	CREATE_ONE_END (name)
 }
 
@@ -53,14 +51,14 @@ FORM (GRAPHICS_DataModeler_speckle, U"DataModeler: Speckle", nullptr) {
 	REAL (ymin, U"left Y range", U"0.0")
 	REAL (ymax, U"right Y range", U"0.0")
 	BOOLEAN (errorBars, U"Draw error bars", 1)
-	REAL (barWidth_mm, U"Bar width (mm)", U"1.0")
-	REAL (xOffset_mm, U"Horizontal offset (mm)", U"0.0")
+	REAL (barWidth_wc, U"Bar width (wc)", U"1.0")
+	REAL (xOffset_wc, U"Horizontal offset (wc)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
 	integer order = 6;
 	GRAPHICS_EACH (DataModeler)
-		DataModeler_speckle (me, GRAPHICS, xmin, xmax,ymin, ymax, 0, order + 1, errorBars, barWidth_mm, xOffset_mm, garnish);
+		DataModeler_speckle (me, GRAPHICS, xmin, xmax,ymin, ymax, 0, order + 1, errorBars, barWidth_wc, xOffset_wc, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -71,7 +69,7 @@ FORM (GRAPHICS_DataModeler_drawEstimatedTrack, U"DataModeler: Draw estimated tra
 	REAL (ymin, U"left Y range", U"0.0")
 	REAL (ymax, U"right Y range", U"0.0")
 	INTEGER (order, U"Order of polynomials for estimation", U"3")
-	REAL (xOffset, U"Horizontal offset (mm)", U"0.0")
+	REAL (xOffset, U"Horizontal offset (wc)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
@@ -90,7 +88,7 @@ DIRECT (INTEGER_DataModeler_getNumberOfParameters) {
 DIRECT (INTEGER_DataModeler_getNumberOfFixedParameters) {
 	INTEGER_ONE (DataModeler)
 		integer result = DataModeler_getNumberOfFixedParameters (me);
-	INTEGER_ONE_END (U"(= number of parameters)")
+	INTEGER_ONE_END (U" (= number of fixed parameters)")
 }
 
 FORM (REAL_DataModeler_getParameterValue, U"DataModeler: Get parameter value", nullptr) {
@@ -107,9 +105,9 @@ FORM (INFO_DataModeler_getParameterStatus, U"DataModeler: Get parameter status",
 	OK
 DO
 	STRING_ONE (DataModeler)
-		int status = DataModeler_getParameterStatus (me, parameterNumber);
-		conststring32 result = ( status == DataModeler_PARAMETER_FREE ? U"Free" :
-			status == DataModeler_PARAMETER_FIXED ? U"Fixed" : U"Undefined" );
+		kDataModelerParameter status = DataModeler_getParameterStatus (me, parameterNumber);
+		conststring32 result = ( status == kDataModelerParameter::FREE ? U"Free" :
+			status == kDataModelerParameter::FIXED_ ? U"Fixed" : U"Undefined" );
 	STRING_ONE_END
 }
 
@@ -123,8 +121,8 @@ DO
 }
 
 FORM (REAL_DataModeler_getVarianceOfParameters, U"DataModeler: Get variance of parameters", nullptr) {
-	INTEGER (fromParameter, U"left Parameter range", U"0")
-	INTEGER (toParameter, U"right Parameter range", U"0")
+	NATURAL (fromParameter, U"left Parameter range", U"1")
+	INTEGER (toParameter, U"right Parameter range", U"0 (=all)")
 	OK
 DO
 	integer nofp;
@@ -165,7 +163,7 @@ DIRECT (REAL_DataModeler_getResidualSumOfSquares) {
 
 DIRECT (REAL_DataModeler_getDataStandardDeviation) {
 	NUMBER_ONE (DataModeler)
-		double result = DataModeler_estimateSigmaY (me);
+		double result = DataModeler_getDataStandardDeviation (me);
 	NUMBER_ONE_END (U"")
 }
 
@@ -201,8 +199,8 @@ FORM (INFO_DataModeler_getDataPointStatus, U"DataModeler: Get data point status"
 	OK
 DO
 	STRING_ONE (DataModeler)
-		int status = DataModeler_getDataPointStatus (me, index);
-		conststring32 result = ( status == DataModeler_DATA_INVALID ? U"Invalid" : U"Valid" );
+		kDataModelerData status = DataModeler_getDataPointStatus (me, index);
+		conststring32 result = ( status == kDataModelerData::INVALID ? U"Invalid" : U"Valid" );
 	STRING_ONE_END
 }
 
@@ -213,17 +211,10 @@ DIRECT (REAL_DataModeler_getCoefficientOfDetermination) {
 }
 
 
-FORM (INFO_DataModeler_reportChiSquared, U"DataModeler: Report chi squared", nullptr) {
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Sigma")
-		OPTION (U"Relative")
-		OPTION (U"Sqrt sigma")
-	OK
-DO
+DIRECT (INFO_DataModeler_reportChiSquared) {
 	INFO_ONE (DataModeler)
 		MelderInfo_open();
-		DataModeler_reportChiSquared (me, weighDataType);
+		DataModeler_reportChiSquared (me);
 		MelderInfo_close();
 	INFO_ONE_END
 }
@@ -236,11 +227,7 @@ DIRECT (REAL_DataModeler_getDegreesOfFreedom) {
 
 
 FORM (MODIFY_DataModeler_setDataWeighing, U"DataModeler: Set data weighing", nullptr) {
-	OPTIONMENUx (weighDataType, U"Weigh data", 1, 0)
-		OPTION (U"Equally")
-		OPTION (U"Sigma")
-		OPTION (U"Relative")
-		OPTION (U"Sqrt sigma")
+	OPTIONMENU_ENUM (kDataModelerWeights, weighDataType, U"Weigh data", kDataModelerWeights::DEFAULT)
 	OK
 DO
 	MODIFY_EACH (DataModeler)
@@ -260,13 +247,11 @@ DO
 FORM (MODIFY_DataModeler_setParameterValue, U"DataModeler: Set parameter value", nullptr) {
 	NATURAL (parameterNumber, U"Parameter number", U"1")
 	REAL (value, U"Value", U"0.0")
-	OPTIONMENU (parameterStatus, U"Status", 1)
-		OPTION (U"Free")
-		OPTION (U"Fixed")
+	OPTIONMENU_ENUM (kDataModelerParameter, parameterStatus, U"Status", kDataModelerParameter::DEFAULT)
 	OK
 DO
 	MODIFY_EACH (DataModeler)
-		DataModeler_setParameterValue (me, parameterNumber, value, parameterStatus - 1);
+		DataModeler_setParameterValue (me, parameterNumber, value, parameterStatus);
 	MODIFY_EACH_END
 }
 
@@ -296,7 +281,7 @@ FORM (MODIFY_DataModeler_setDataPointStatus, U"DataModeler: Set data point statu
 		OPTION (U"Invalid")
 	OK
 DO
-	int status = dataStatus == 2 ? DataModeler_DATA_INVALID : DataModeler_DATA_VALID;
+	kDataModelerData status = dataStatus == 2 ? kDataModelerData::INVALID : kDataModelerData::VALID;
 	MODIFY_EACH (DataModeler)
 		DataModeler_setDataPointStatus (me, index, status);
 	MODIFY_EACH_END
@@ -355,12 +340,9 @@ DIRECT (NEW_DataModeler_to_Covariance_parameters) {
 	CONVERT_EACH_END (my name.get())
 }
 
-FORM (NEW_DataModeler_to_Table_zscores, U"DataModeler: To Table (z-scores)", nullptr) {
-	BOOLEAN (useSigmaY, U"Use sigmas on y-values", 1)
-	OK
-DO
+DIRECT (NEW_DataModeler_to_Table_zscores) {
 	CONVERT_EACH (DataModeler)
-		autoTable result = DataModeler_to_Table_zscores (me, useSigmaY);
+		autoTable result = DataModeler_to_Table_zscores (me);
 	CONVERT_EACH_END (my name.get(), U"_z");
 }
 
@@ -370,16 +352,11 @@ FORM (NEW_Formant_to_FormantModeler, U"Formant: To FormantModeler", nullptr) {
 	REAL (toTime, U"right End time", U"0.1")
 	NATURAL (numberOfFormants, U"Number of formants", U"3")
 	INTEGER (order, U"Order of polynomials", U"3")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
 	OK
 DO
 	Melder_require (order >= 0, U"The order should be at least zero.");
 	CONVERT_EACH (Formant)
-		autoFormantModeler result = Formant_to_FormantModeler (me, fromTime, toTime, numberOfFormants, order + 1, weighDataType);
+		autoFormantModeler result = Formant_to_FormantModeler (me, fromTime, toTime, numberOfFormants, order + 1);
 	CONVERT_EACH_END (my name.get(), U"_o", order);
 }
 
@@ -389,11 +366,7 @@ FORM (NEW1_Formants_extractSmoothestPart, U"Formants: Extract smoothest part", U
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
 	LABEL (U"Use bandwidths to model the formant tracks:")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Zero parameter values whose range include zero:")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
@@ -404,7 +377,7 @@ DO
 		iam (Formant);
 		formants. addItem_ref (me);
 	}
-	integer index = Formants_getSmoothestInInterval (& formants, fromTime, toTime, numberOfFormantTracks, order + 1, weighDataType, 0, numberOfSigmas, power, 1.0, 1.0, 1.0, 1.0, 1.0);
+	integer index = Formants_getSmoothestInInterval (& formants, fromTime, toTime, numberOfFormantTracks, order + 1, weighFormants, 0, numberOfSigmas, power, 1.0, 1.0, 1.0, 1.0, 1.0);
 	// next code is necessary to get the Formant at postion index selected and to get its name
 	integer iselected = 0;
 	Formant him = nullptr;
@@ -425,11 +398,7 @@ FORM (NEW1_Formants_extractSmoothestPart_constrained, U"Formants: Extract smooth
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
 	LABEL (U"Use bandwidths to model the formant tracks:")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Zero parameter values whose range include zero:")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
@@ -446,7 +415,7 @@ DO
 		iam (Formant);
 		formants. addItem_ref (me);
 	}
-	integer index = Formants_getSmoothestInInterval (& formants, fromTime, toTime, numberOfFormantTracks, order + 1, weighDataType, 1, numberOfSigmas, power, minimumF1, maximumF1, minimumF2, maximumF2, minimumF3);
+	integer index = Formants_getSmoothestInInterval (& formants, fromTime, toTime, numberOfFormantTracks, order + 1, weighFormants, 1, numberOfSigmas, power, minimumF1, maximumF1, minimumF2, maximumF2, minimumF3);
 	// next code is necessary to get the Formant at postion index selected and to get its name
 	integer iselected = 0;
 	Formant him = nullptr;
@@ -470,13 +439,13 @@ FORM (GRAPHICS_FormantModeler_drawEstimatedTracks, U"FormantModeler: Draw estima
 	NATURAL (fromFormant, U"left Formant range", U"1")
 	NATURAL (toFormant, U"right Formant range", U"3")
 	INTEGER (order, U"Order of polynomials for estimation", U"3")
-	REAL (xOffset_mm, U"Horizontal offset (mm)", U"0.0")
+	REAL (xOffset, U"Horizontal offset (s)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
 	Melder_require (order >= 0, U"The order should be at least zero.");
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_drawTracks (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 1, order + 1, xOffset_mm, garnish);
+		FormantModeler_drawTracks (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 1, order + 1, xOffset, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -486,13 +455,13 @@ FORM (GRAPHICS_FormantModeler_drawTracks, U"FormantModeler: Draw tracks", nullpt
 	REAL (maximumFrequency, U"Maximum frequency (Hz)", U"5500.0")
 	NATURAL (fromFormant, U"left Formant range", U"1")
 	NATURAL (toFormant, U"right Formant range", U"3")
-	REAL (xOffset_mm, U"Horizontal offset (mm)", U"0.0")
+	REAL (xOffset_s, U"Horizontal offset (s)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
 	integer order = 6;
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_drawTracks (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 0, order + 1, xOffset_mm, garnish);
+		FormantModeler_drawTracks (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 0, order + 1, xOffset_s, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -503,14 +472,14 @@ FORM (GRAPHICS_FormantModeler_speckle, U"FormantModeler: Speckle", nullptr) {
 	NATURAL (fromFormant, U"left Formant range", U"1")
 	NATURAL (toFormant, U"right Formant range", U"3")
 	BOOLEAN (errorBars, U"Draw error bars", true)
-	REAL (barWidth_mm, U"Bar width (mm)", U"1.0")
-	REAL (xOffset_mm, U"Horizontal offset (mm)", U"0.0")
+	REAL (barWidth_s, U"Bar width (s)", U"1.0")
+	REAL (xOffset_s, U"Horizontal offset (s)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
 	integer order = 6;
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_speckle (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 0, order + 1, errorBars, barWidth_mm, xOffset_mm, garnish);
+		FormantModeler_speckle (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, 0, order + 1, errorBars, barWidth_s, xOffset_s, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -521,19 +490,14 @@ FORM (GRAPHICS_FormantModeler_drawOutliersMarked, U"FormantModeler: Draw outlier
 	NATURAL (fromFormant, U"left Formant range", U"1")
 	NATURAL (toFormant, U"right Formant range", U"3")
 	POSITIVE (numberOfSigmas, U"Number of sigmas", U"3.0")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
 	WORD (mark_string, U"Mark", U"o")
 	POSITIVE (fontSize, U"Mark font size", U"12")
-	REAL (xOffset_mm, U"Horizontal offset (mm)", U"0.0")
+	REAL (xOffset_s, U"Horizontal offset (s)", U"0.0")
 	BOOLEAN (garnish, U"Garnish", false)
 	OK
 DO
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_drawOutliersMarked (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, numberOfSigmas, weighDataType, mark_string, fontSize, xOffset_mm, garnish);
+		FormantModeler_drawOutliersMarked (me, GRAPHICS, fromTime, toTime, maximumFrequency, fromFormant, toFormant, numberOfSigmas, mark_string, fontSize, xOffset_s, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -542,10 +506,7 @@ FORM (GRAPHICS_FormantModeler_drawVariancesOfShiftedTracks, U"FormantModeler: Dr
 	REAL (toTime, U"right Time range (s)", U"0.0")
 	REAL (fromVariance, U"left Variance range", U"0.0")
 	REAL (toVariance, U"right Variance range", U"0.0")
-	OPTIONMENU (shiftTracks, U"Shift tracks", 1)
-		OPTION (U"No")
-		OPTION (U"Up")
-		OPTION (U"Down")
+	OPTIONMENU_ENUM (kFormantModelerTrackShift, shiftTracks, U"Shift tracks", kFormantModelerTrackShift::DEFAULT)
 	NATURAL (fromFormant, U"left Formant range", U"1")
 	NATURAL (toFormant, U"right Formant range", U"4")
 	BOOLEAN (garnish, U"Garnish", true)
@@ -561,27 +522,17 @@ FORM (GRAPHICS_FormantModeler_drawCumulativeChisqScores, U"FormantModeler: Draw
 	REAL (toTime, U"right Time range (s)", U"0.0")
 	REAL (fromChisq, U"left Chisq range", U"0.0")
 	REAL (toChisq, U"right Chisq range", U"0.0")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
 	BOOLEAN (garnish, U"Garnish", true)
 	OK
 DO
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_drawCumulativeChiScores (me, GRAPHICS, fromTime, toTime, fromChisq, toChisq, weighDataType, garnish);
+		FormantModeler_drawCumulativeChiScores (me, GRAPHICS, fromTime, toTime, fromChisq, toChisq, garnish);
 	GRAPHICS_EACH_END
 }
 
 
 FORM (GRAPHICS_FormantModeler_normalProbabilityPlot, U"FormantModeler: Normal probability plot", nullptr) {
 	NATURAL (formantNumber, U"Formant number", U"1")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
 	NATURAL (numberOfQuantiles, U"Number of quantiles", U"100")
 	REAL (numberOfSigmas, U"Number of sigmas", U"0.0")
 	POSITIVE (fontSize, U"Label size", U"12")
@@ -590,7 +541,7 @@ FORM (GRAPHICS_FormantModeler_normalProbabilityPlot, U"FormantModeler: Normal pr
 	OK
 DO
 	GRAPHICS_EACH (FormantModeler)
-		FormantModeler_normalProbabilityPlot (me, GRAPHICS, formantNumber, weighDataType, numberOfQuantiles, numberOfSigmas, fontSize, label, garnish);
+		FormantModeler_normalProbabilityPlot (me, GRAPHICS, formantNumber, numberOfQuantiles, numberOfSigmas, fontSize, label, garnish);
 	GRAPHICS_EACH_END
 }
 
@@ -599,7 +550,6 @@ FORM (GRAPHICS_FormantModeler_drawBasisFunction, U"FormantModeler: Draw basis fu
 	REAL (toTime, U"right Time range (s)", U"0.0")
 	REAL (ymin, U"left Amplitude range (Hz)", U"0.0")
 	REAL (ymax, U"right Amplitude range (Hz)", U"5500.0")
-	//integer iterm, bool scaled, integer numberOfPoints, bool garnish
 	NATURAL (formantNumber, U"Formant number", U"1")
 	NATURAL (basisFunctionIndex, U"Basis function", U"2")
 	BOOLEAN (scale, U"Scale function with parameter value", false)
@@ -650,8 +600,8 @@ FORM (INFO_FormantModeler_getDataPointStatus, U"FormantModeler: Get data point s
 	OK
 DO
 	INTEGER_ONE (FormantModeler)
-		int status = FormantModeler_getDataPointStatus (me, formantNumber, index);
-		conststring32 result = ( status == DataModeler_DATA_INVALID ? U"Invalid" : U"Valid" );
+		kDataModelerData status = FormantModeler_getDataPointStatus (me, formantNumber, index);
+		conststring32 result = ( status == kDataModelerData::INVALID ? U"Invalid" : U"Valid" );
 	INTEGER_ONE_END (U"")
 }
 
@@ -712,9 +662,9 @@ FORM (INFO_FormantModeler_getParameterStatus, U"FormantModeler: Get parameter st
 	OK
 DO
 	STRING_ONE (FormantModeler)
-		int status = FormantModeler_getParameterStatus (me, formantNumber, parameterNumber);
+		kDataModelerParameter status = FormantModeler_getParameterStatus (me, formantNumber, parameterNumber);
 		conststring32 result = Melder_cat (
-			status == DataModeler_PARAMETER_FREE ? U"Free" : status == DataModeler_PARAMETER_FIXED ? U"Fixed" : U"Undefined",
+			status == kDataModelerParameter::FREE ? U"Free" : status == kDataModelerParameter::FIXED_ ? U"Fixed" : U"Undefined",
 			U" (= status of parameter ", parameterNumber, U" for F", formantNumber, U")"
 		);
 	STRING_ONE_END
@@ -774,17 +724,10 @@ DO
 	NUMBER_ONE_END (U" Hz (= standard deviation of F", formantNumber, U")")
 }
 
-FORM (INFO_FormantModeler_reportChiSquared, U"FormantModeler: Report chi squared", nullptr) {
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
-	OK
-DO
+DIRECT (INFO_FormantModeler_reportChiSquared) {
 	INFO_ONE (FormantModeler)
 		MelderInfo_open();
-		FormantModeler_reportChiSquared (me, weighDataType);
+		FormantModeler_reportChiSquared (me);
 		MelderInfo_close();
 	INFO_ONE_END
 }
@@ -836,18 +779,14 @@ DO
 	NUMBER_ONE_END (U" (= formants constraints factor)");
 }
 
-FORM (MODIFY_FormantModeler_setDataWeighing, U"FormantModeler: Set data weighing", nullptr) {
+FORM (MODIFY_FormantModeler_setFormantWeighing, U"FormantModeler: Set data weighing", nullptr) {
 	INTEGER (fromFormant, U"left Formant range", U"0")
 	INTEGER (toFormant, U"right Formant range", U"0")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh data", kFormantModelerWeights::DEFAULT)
 	OK
 DO
 	MODIFY_EACH (FormantModeler)
-		FormantModeler_setDataWeighing (me, fromFormant, toFormant, weighDataType);
+		FormantModeler_setDataWeighing (me, fromFormant, toFormant, weighFormants);
 	MODIFY_EACH_END
 }
 
@@ -920,12 +859,9 @@ DO
 FORM (MODIFY_FormantModeler_setDataPointStatus, U"FormantModeler: Set data point status", nullptr) {
 	NATURAL (formantNumber, U"Formant index", U"1")
 	NATURAL (dataNumber, U"Data index", U"1")
-	OPTIONMENU (dataStatus, U"Status", 1)
-		OPTION (U"Valid")
-		OPTION (U"Invalid")
+	OPTIONMENU_ENUM (kDataModelerData, status, U"Status", kDataModelerData::DEFAULT)
 	OK
 DO
-	int status = dataStatus == 2 ? DataModeler_DATA_INVALID : DataModeler_DATA_VALID;
 	MODIFY_EACH (FormantModeler)
 		FormantModeler_setDataPointStatus (me, formantNumber,  dataNumber, status);
 	MODIFY_EACH_END
@@ -955,22 +891,18 @@ DO
 	CONVERT_EACH_END (my name.get(), U"_", formantNumber)
 }
 
-FORM (NEW_FormantModeler_to_Table_zscores, U"", nullptr) {
-	BOOLEAN (useBandwidth, U"Bandwidths as standard deviation", true)
-	OK
-DO
+DIRECT (NEW_FormantModeler_to_Table_zscores) {
 	CONVERT_EACH (FormantModeler)
-		autoTable result = FormantModeler_to_Table_zscores (me, useBandwidth);
+		autoTable result = FormantModeler_to_Table_zscores (me);
 	CONVERT_EACH_END (my name.get(), U"_z")
 }
 
 FORM (NEW_FormantModeler_to_FormantModeler_processOutliers, U"", nullptr) {
 	POSITIVE (numberOfSigmas, U"Number of sigmas", U"3.0")
-	BOOLEAN (useBandwidth, U"Bandwidths as standard deviation", true)
 	OK
 DO
 	CONVERT_EACH (FormantModeler)
-		autoFormantModeler result = FormantModeler_processOutliers (me, numberOfSigmas, useBandwidth);
+		autoFormantModeler result = FormantModeler_processOutliers (me, numberOfSigmas);
 	CONVERT_EACH_END (my name.get(), U"_outliers");
 }
 
@@ -986,7 +918,7 @@ DIRECT (WINDOW_OptimalCeilingTier_edit) {
 
 
 /*************************** PitchModeler *************************************/
-
+/*
 FORM (NEW_Pitch_to_PitchModeler, U"Pitch: To PitchModeler", nullptr) {
 	REAL (fromTime, U"left Start time (s)", U"0.0")
 	REAL (toTime, U"right End time (s)", U"0.1")
@@ -1011,6 +943,7 @@ DO
 		PitchModeler_draw (me, GRAPHICS, fromTime, toTime, fromFrequency, toFrequency, order + 1,  garnish);
 	GRAPHICS_EACH_END
 }
+*/
 
 FORM (REAL_Sound_getOptimalFormantCeiling, U"Sound: Get optimal formant ceiling", nullptr) {
 	REAL (fromTime, U"left Time range (s)", U"0.1")
@@ -1023,18 +956,14 @@ FORM (REAL_Sound_getOptimalFormantCeiling, U"Sound: Get optimal formant ceiling"
 	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks in model", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Make parameters that include zero in their confidence region zero")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
 	OK
 DO
 	NUMBER_ONE (Sound)
-		double result = Sound_getOptimalFormantCeiling (me, fromTime, toTime, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighDataType, numberOfSigmas, power);
+		double result = Sound_getOptimalFormantCeiling (me, fromTime, toTime, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighFormants, numberOfSigmas, power);
 	NUMBER_ONE_END (U" Hz");
 }
 
@@ -1049,11 +978,7 @@ FORM (NEW_Sound_to_Formant_interval, U"Sound: To Formant (interval)", nullptr) {
 	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks in model", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Make parameters that include zero in their confidence region zero")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
@@ -1061,7 +986,7 @@ FORM (NEW_Sound_to_Formant_interval, U"Sound: To Formant (interval)", nullptr) {
 DO
 	CONVERT_EACH (Sound)
 		double ceiling;
-		autoFormant result = Sound_to_Formant_interval (me, fromTime, toTime, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighDataType, numberOfSigmas, power, 0, 1, 1, 1, 1, 1, &ceiling);
+		autoFormant result = Sound_to_Formant_interval (me, fromTime, toTime, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighFormants, numberOfSigmas, power, 0, 1, 1, 1, 1, 1, &ceiling);
 	CONVERT_EACH_END (my name.get(), U"_", Melder_fixed (ceiling, 0))
 }
 
@@ -1076,11 +1001,7 @@ FORM (NEW_Sound_to_Formant_interval_constrained, U"Sound: To Formant (interval,
 	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks in model", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Make parameters that include zero in their confidence region zero")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
@@ -1094,7 +1015,7 @@ FORM (NEW_Sound_to_Formant_interval_constrained, U"Sound: To Formant (interval,
 DO
 	CONVERT_EACH (Sound)
 		double ceiling;
-		autoFormant result = Sound_to_Formant_interval (me, fromTime, toTime, windowLength, timeStep, fromFrequency,  toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighDataType, numberOfSigmas, power, 1, minimumF1, maximumF1, minimumF2, maximumF2, minimumF3, & ceiling);
+		autoFormant result = Sound_to_Formant_interval (me, fromTime, toTime, windowLength, timeStep, fromFrequency,  toFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighFormants, numberOfSigmas, power, 1, minimumF1, maximumF1, minimumF2, maximumF2, minimumF3, & ceiling);
 	CONVERT_EACH_END (my name.get(), U"_", Melder_fixed (ceiling, 0));
 }
 
@@ -1109,11 +1030,7 @@ FORM (NEW_Sound_to_Formant_interval_constrained_robust, U"Sound: To Formant (int
 	POSITIVE (preEmphasisFrequency, U"Pre-emphasis from (Hz)", U"50.0")
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks in model", U"4")
 	INTEGER (order, U"Order of polynomials", U"3")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Make parameters that include zero in their confidence region zero")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
@@ -1127,7 +1044,7 @@ FORM (NEW_Sound_to_Formant_interval_constrained_robust, U"Sound: To Formant (int
 DO
 	CONVERT_EACH (Sound)
 		double ceiling;
-		autoFormant result = Sound_to_Formant_interval_robust (me, fromTime, toTime, windowLength, timeStep, fromFrequency, fromFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighDataType, numberOfSigmas, power, 1, minimumF1, maximumF1, minimumF2, minimumF2, minimumF3, &ceiling);
+		autoFormant result = Sound_to_Formant_interval_robust (me, fromTime, toTime, windowLength, timeStep, fromFrequency, fromFrequency, numberOfFrequencySteps, preEmphasisFrequency, numberOfFormantTracks, order + 1, weighFormants, numberOfSigmas, power, 1, minimumF1, maximumF1, minimumF2, minimumF2, minimumF3, &ceiling);
 	CONVERT_EACH_END (my name.get(), U"_", Melder_fixed (ceiling, 0))
 }
 
@@ -1141,18 +1058,14 @@ FORM (NEW_Sound_to_OptimalCeilingTier, U"", nullptr) {
 	REAL (smoothingWindow_s, U"Formant smoothing window (s)", U"0.05")
 	NATURAL (numberOfFormantTracks, U"Number of formant tracks in model", U"4")
 	INTEGER (order, U"Order of polynomials", U"2")
-	OPTIONMENUx (weighDataType, U"Weigh data", 2, 0)
-		OPTION (U"Equally")
-		OPTION (U"Bandwidth")
-		OPTION (U"Bandwidth / frequency")
-		OPTION (U"Sqrt bandwidth")
+	OPTIONMENU_ENUM (kFormantModelerWeights, weighFormants, U"Weigh formants", kFormantModelerWeights::DEFAULT)
 	LABEL (U"Make parameters that include zero in their confidence region zero")
 	REAL (numberOfSigmas, U"Number of sigmas", U"1.0")
 	REAL (power, U"Parameter variance power", U"1.5")
 	OK
 DO
 	CONVERT_EACH (Sound)
-		autoOptimalCeilingTier result = Sound_to_OptimalCeilingTier (me, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, smoothingWindow_s, numberOfFormantTracks, order + 1, weighDataType, numberOfSigmas, power);
+		autoOptimalCeilingTier result = Sound_to_OptimalCeilingTier (me, windowLength, timeStep, fromFrequency, toFrequency, numberOfFrequencySteps, preEmphasisFrequency, smoothingWindow_s, numberOfFormantTracks, order + 1, weighFormants, numberOfSigmas, power);
 	CONVERT_EACH_END (my name.get());
 }
 
@@ -1162,8 +1075,7 @@ FORM (NEW_Table_to_DataModeler, U"", nullptr) {
 	WORD (columnWithX_string, U"Column with X data", U"")
 	WORD (columnWithY_string, U"Column with Y data", U"")
 	WORD (columnEithSigma_string, U"Column with sigmas", U"")
-	OPTIONMENU (functionType, U"Model functions", 1)
-		OPTION (U"Legendre polynomials")
+	OPTIONMENU_ENUM (kDataModelerFunction, type, U"Basis functions", kDataModelerFunction::DEFAULT)		
 	INTEGER (maximumOrder, U"Maximum order", U"3")
 	OK
 DO
@@ -1171,14 +1083,14 @@ DO
 		integer xcolumn = Table_getColumnIndexFromColumnLabel (me, columnWithX_string);
 		integer ycolumn = Table_getColumnIndexFromColumnLabel (me, columnWithY_string);
 		integer scolumn = Table_findColumnIndexFromColumnLabel (me, columnEithSigma_string);
-		autoDataModeler result = Table_to_DataModeler (me, xmin, xmax, xcolumn, ycolumn, scolumn, maximumOrder + 1, functionType);
+		autoDataModeler result = Table_to_DataModeler (me, xmin, xmax, xcolumn, ycolumn, scolumn, maximumOrder + 1, type);
 	CONVERT_EACH_END (my name.get())
 }
 
 void praat_DataModeler_init ();
 void praat_DataModeler_init () {
-	Thing_recognizeClassesByName (classDataModeler, classFormantModeler, classOptimalCeilingTier, classOptimalCeilingTierEditor, classPitchModeler, nullptr);
-
+	Thing_recognizeClassesByName (classDataModeler, classFormantModeler, classOptimalCeilingTier, classOptimalCeilingTierEditor, nullptr);
+	
 	praat_addMenuCommand (U"Objects", U"New", U"Create simple DataModeler...", U"Create ISpline...", praat_HIDDEN + praat_DEPTH_1, NEW1_DataModeler_createSimple);
 
 	praat_addAction1 (classDataModeler, 0, U"Speckle...", 0, 0, GRAPHICS_DataModeler_speckle);
@@ -1204,7 +1116,7 @@ void praat_DataModeler_init () {
 		praat_addAction1 (classDataModeler, 0, U"Get residual sum of squares", 0, 1, REAL_DataModeler_getResidualSumOfSquares);
 		praat_addAction1 (classDataModeler, 0, U"Get data standard deviation", 0, 1, REAL_DataModeler_getDataStandardDeviation);
 		praat_addAction1 (classDataModeler, 0, U"Get coefficient of determination", 0, 1, REAL_DataModeler_getCoefficientOfDetermination);
-		praat_addAction1 (classDataModeler, 0, U"Report chi squared...", 0, 1, INFO_DataModeler_reportChiSquared);
+		praat_addAction1 (classDataModeler, 0, U"Report chi squared", 0, 1, INFO_DataModeler_reportChiSquared);
 		praat_addAction1 (classDataModeler, 0, U"Get degrees of freedom", 0, 1, REAL_DataModeler_getDegreesOfFreedom);
 
 		praat_addAction1 (classDataModeler, 1, U"Modify -", 0, 0, 0);
@@ -1223,7 +1135,7 @@ void praat_DataModeler_init () {
 	praat_addAction1 (classDataModeler, 0, U"Fit model", 0, 0, MODIFY_DataModeler_fitModel);
 	
 	praat_addAction1 (classDataModeler, 0, U"To Covariance (parameters)", 0, 0, NEW_DataModeler_to_Covariance_parameters);
-	praat_addAction1 (classDataModeler, 0, U"To Table (z-scores)...", 0, 0, NEW_DataModeler_to_Table_zscores);
+	praat_addAction1 (classDataModeler, 0, U"To Table (z-scores)", 0, 0, NEW_DataModeler_to_Table_zscores);
 
 	praat_addAction1 (classFormant, 0, U"To FormantModeler...", U"To LPC...", praat_HIDDEN, NEW_Formant_to_FormantModeler);
 	praat_addAction1 (classFormant, 0, U"Extract smoothest part...", 0, praat_HIDDEN, NEW1_Formants_extractSmoothestPart);
@@ -1260,14 +1172,14 @@ void praat_DataModeler_init () {
 		praat_addAction1 (classFormantModeler, 0, U"Get residual sum of squares...", 0, 1, REAL_FormantModeler_getResidualSumOfSquares);
 		praat_addAction1 (classFormantModeler, 0, U"Get formant standard deviation...", 0, 1, REAL_FormantModeler_getFormantStandardDeviation);
 		praat_addAction1 (classFormantModeler, 0, U"Get coefficient of determination...", 0, 1, REAL_FormantModeler_getCoefficientOfDetermination);
-		praat_addAction1 (classFormantModeler, 0, U"Report chi squared...", 0, 1, INFO_FormantModeler_reportChiSquared);
+		praat_addAction1 (classFormantModeler, 0, U"Report chi squared", 0, 1, INFO_FormantModeler_reportChiSquared);
 		praat_addAction1 (classFormantModeler, 0, U"Get degrees of freedom...", 0, 1, REAL_FormantModeler_getDegreesOfFreedom);
 		praat_addAction1 (classFormantModeler, 0, U"Get smoothness value...", 0, 1, REAL_FormantModeler_getSmoothnessValue);
 		praat_addAction1 (classFormantModeler, 0, U"Get average distance between tracks...", 0, 1, REAL_FormantModeler_getAverageDistanceBetweenTracks);
 		praat_addAction1 (classFormantModeler, 0, U"Get formants constraints factor...", 0, 1, REAL_FormantModeler_getFormantsConstraintsFactor);
 
 	praat_addAction1 (classFormantModeler, 1, U"Modify -", 0, 0, 0);
-		praat_addAction1 (classFormantModeler, 0, U"Set data weighing...", 0, 1, MODIFY_FormantModeler_setDataWeighing);
+		praat_addAction1 (classFormantModeler, 0, U"Set data weighing...", 0, 1, MODIFY_FormantModeler_setFormantWeighing);
 		praat_addAction1 (classFormantModeler, 0, U"Set tolerance...", 0, 1, MODIFY_FormantModeler_setTolerance);
 		praat_addAction1 (classFormantModeler, 1, U"-- set parameter values --", 0, 1, 0);
 		praat_addAction1 (classFormantModeler, 0, U"Set parameter value fixed...", 0, 1, MODIFY_FormantModeler_setParameterValueFixed);
@@ -1282,15 +1194,15 @@ void praat_DataModeler_init () {
 	
 	
 	praat_addAction1 (classFormantModeler, 0, U"To Covariance (parameters)...", 0, 0, NEW_FormantModeler_to_Covariance_parameters);
-	praat_addAction1 (classFormantModeler, 0, U"To Table (z-scores)...", 0, 0, NEW_FormantModeler_to_Table_zscores);
+	praat_addAction1 (classFormantModeler, 0, U"To Table (z-scores)", 0, 0, NEW_FormantModeler_to_Table_zscores);
 	praat_addAction1 (classFormantModeler, 0, U"To FormantModeler (process outliers)...", 0, 0, NEW_FormantModeler_to_FormantModeler_processOutliers);
 	praat_addAction1 (classFormantModeler, 0, U"Extract DataModeler...", 0, 0, NEW_FormantModeler_extractDataModeler);
 
 	praat_addAction1 (classOptimalCeilingTier, 1, U"View & Edit", 0, praat_ATTRACTIVE | praat_NO_API, WINDOW_OptimalCeilingTier_edit);
 	
-	praat_addAction1 (classPitch, 0, U"To PitchModeler...", U"To PointProcess", praat_HIDDEN, NEW_Pitch_to_PitchModeler);
+	//praat_addAction1 (classPitch, 0, U"To PitchModeler...", U"To PointProcess", praat_HIDDEN, NEW_Pitch_to_PitchModeler);
 
-	praat_addAction1 (classPitchModeler, 0, U"Draw...", 0, 0, GRAPHICS_PitchModeler_draw);
+	//praat_addAction1 (classPitchModeler, 0, U"Draw...", 0, 0, GRAPHICS_PitchModeler_draw);
 
 	praat_addAction1 (classSound, 0, U"Get optimal formant ceiling...", U"Get intensity (dB)", praat_DEPTH_1 | praat_HIDDEN, REAL_Sound_getOptimalFormantCeiling);
 	praat_addAction1 (classSound, 0, U"To Formant (interval)...", U"To Formant (robust)...", praat_DEPTH_2 | praat_HIDDEN, NEW_Sound_to_Formant_interval);
