@@ -1,6 +1,6 @@
 /* FunctionEditor.cpp
  *
- * Copyright (C) 1992-2019 Paul Boersma
+ * Copyright (C) 1992-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ static void drawNow (FunctionEditor me) {
 	if (endIsVisible && my endSelection != my startSelection)
 		my marker [++ my numberOfMarkers] = my endSelection;
 	my marker [++ my numberOfMarkers] = my endWindow;
-	VECsort_inplace (VEC (my marker, my numberOfMarkers));
+	VECsort_inplace (VEC (& my marker [1], my numberOfMarkers));
 
 	/*
 		Update rectangles.
@@ -131,7 +131,7 @@ static void drawNow (FunctionEditor me) {
 	my rect [1]. left = my functionViewerLeft + MARGIN;
 	my rect [1]. right = my functionViewerRight - MARGIN;
 	my rect [1]. bottom = BOTTOM_MARGIN + space;
-	my rect [1]. top = BOTTOM_MARGIN + space * (my numberOfMarkers > 1 ? 2 : 3);
+	my rect [1]. top = BOTTOM_MARGIN + space * ( my numberOfMarkers > 1 ? 2 : 3 );
 
 	/*
 		2: rectangle for left from visible part.
@@ -1759,7 +1759,7 @@ void FunctionEditor_ungroup (FunctionEditor me) {
 }
 
 void FunctionEditor_drawRangeMark (FunctionEditor me, double yWC, conststring32 yWC_string, conststring32 units, int verticalAlignment) {
-	static MelderString text { };
+	static MelderString text;
 	MelderString_copy (& text, yWC_string, units);
 	double textWidth = Graphics_textWidth (my graphics.get(), text.string) + Graphics_dxMMtoWC (my graphics.get(), 0.5);
 	Graphics_setColour (my graphics.get(), Melder_BLUE);
@@ -1793,7 +1793,7 @@ void FunctionEditor_insertCursorFunctionValue (FunctionEditor me, double yWC, co
 	} else if (tooLow) {
 		textY = minimum + Graphics_dyMMtoWC (my graphics.get(), 5.0);
 	}
-	static MelderString text { };
+	static MelderString text;
 	MelderString_copy (& text, yWC_string, units);
 	double textWidth = Graphics_textWidth (my graphics.get(), text.string);
 	Graphics_fillCircle_mm (my graphics.get(), my endWindow + textWidth + Graphics_dxMMtoWC (my graphics.get(), 1.5), textY, 1.5);
