@@ -1,6 +1,6 @@
 /* Pitch.cpp
  *
- * Copyright (C) 1992-2009,2011,2012,2014-2019 Paul Boersma
+ * Copyright (C) 1992-2009,2011,2012,2014-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
  */
 
 #include "Pitch.h"
-#include <ctype.h>
 #include "Sound_and_Spectrum.h"
 #include "Matrix_and_Pitch.h"
 
@@ -473,13 +472,6 @@ integer Pitch_getMaxnCandidates (Pitch me) {
 	return result;
 }
 
-static void Pitch_checkFrameNumber (Pitch me, integer frameNumber) {
-    Melder_require (frameNumber >= 1,
-    	U"The frame number should be at least 1, but is ", frameNumber, U" instead.");
-    Melder_require (frameNumber <= my nx,
-    	U"The frame number should at most the number of frames (", my nx, U"), but is ", frameNumber, U" instead.");
-}
-
 autoMAT Pitch_Frame_getAllCandidates (Pitch_Frame me) {
 	integer numberOfCandidates = my nCandidates;
 	autoMAT candidates = newMATraw (2, numberOfCandidates);
@@ -492,7 +484,7 @@ autoMAT Pitch_Frame_getAllCandidates (Pitch_Frame me) {
 
 autoMAT Pitch_getAllCandidatesInFrame (Pitch me, integer frameNumber) {
 	try {
-    	Pitch_checkFrameNumber (me, frameNumber);
+    	my checkIndex (frameNumber);
     	const Pitch_Frame frame = & my frames [frameNumber];
     	return Pitch_Frame_getAllCandidates (frame);
 	} catch (MelderError) {
@@ -959,7 +951,7 @@ static autoTable Pitch_Frame_tabulateCandidates (Pitch_Frame me) {
 }
 
 autoTable Pitch_tabulateCandidatesInFrame (Pitch me, integer frameNumber) {
-	Pitch_checkFrameNumber (me, frameNumber);
+	my checkIndex (frameNumber);
 	const Pitch_Frame frame = & my frames [frameNumber];
 	return Pitch_Frame_tabulateCandidates (frame);
 }
