@@ -1,6 +1,6 @@
 /* Praat_tests.cpp
  *
- * Copyright (C) 2001-2007,2009,2011-2019 Paul Boersma
+ * Copyright (C) 2001-2007,2009,2011-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -396,7 +396,7 @@ int Praat_tests (kPraatTests itest, conststring32 arg1, conststring32 arg2, cons
 		} break;
 		case kPraatTests::TIME_ZERO: {
 			integer size = Melder_atoi (arg2);
-			autoVEC result { size, kTensorInitializationType::RAW };
+			autoVEC result = newVECraw (size);
 			double z = 0.0;
 			for (int64 iteration = 1; iteration <= n; iteration ++) {
 				for (integer i = 1; i <= size; i ++)
@@ -580,7 +580,7 @@ int Praat_tests (kPraatTests itest, conststring32 arg1, conststring32 arg2, cons
 				const autoVEC f { e, 10 };
 				#endif
 				{
-					autoVEC g { 100, kTensorInitializationType::ZERO };
+					autoVEC g = newVECzero (100);
 					g [1] = 3.0;
 					VEC gg = g.get();
 					gg [2] = 4.0;
@@ -594,7 +594,7 @@ int Praat_tests (kPraatTests itest, conststring32 arg1, conststring32 arg2, cons
 					//gg.reset();
 				}
 				{
-					double x [1+2], *px = & x [0];
+					double x [3], *px = & x [0];
 					const double *cpx = px;
 					VEC vx { px, 2 };
 					constVEC cvx { px, 2 };
@@ -639,7 +639,7 @@ int Praat_tests (kPraatTests itest, conststring32 arg1, conststring32 arg2, cons
 					//aa = aac;
 					aa = bb;
 					aac = bbc;
-					bbc.at = bb.at;
+					bbc.cells = bb.cells;
 				}
 			}
 		} break;
