@@ -2,7 +2,7 @@
 #define _Sampled_h_
 /* Sampled.h
  *
- * Copyright (C) 1992-2005,2007,2011,2013-2019 Paul Boersma
+ * Copyright (C) 1992-2005,2007,2011,2013-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ static inline integer Sampled_xToHighIndex    (Sampled me, double x) { return Me
 static inline integer Sampled_xToNearestIndex (Sampled me, double x) { return Melder_iround   ((x - my x1) / my dx + 1.0); }
 
 static inline autoVEC Sampled_listAllXValues (Sampled me) {
-	autoVEC result (my nx, kTensorInitializationType::RAW);
+	autoVEC result = newVECraw (my nx);
 	for (integer i = 1; i <= my nx; i ++)
 		result [i] = my x1 + (i - 1) * my dx;
 	return result;
