@@ -1,6 +1,6 @@
 /* Sound_audio.cpp
  *
- * Copyright (C) 1992-2019 Paul Boersma
+ * Copyright (C) 1992-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,41 +52,6 @@
 	#include <fcntl.h>
 #endif
 
-static int ulaw2linear [] = 
-      { -32124, -31100, -30076, -29052, -28028, -27004, -25980, -24956,
-        -23932, -22908, -21884, -20860, -19836, -18812, -17788, -16764,
-        -15996, -15484, -14972, -14460, -13948, -13436, -12924, -12412,
-        -11900, -11388, -10876, -10364,  -9852,  -9340,  -8828,  -8316,
-         -7932,  -7676,  -7420,  -7164,  -6908,  -6652,  -6396,  -6140,
-         -5884,  -5628,  -5372,  -5116,  -4860,  -4604,  -4348,  -4092,
-         -3900,  -3772,  -3644,  -3516,  -3388,  -3260,  -3132,  -3004,
-         -2876,  -2748,  -2620,  -2492,  -2364,  -2236,  -2108,  -1980,
-         -1884,  -1820,  -1756,  -1692,  -1628,  -1564,  -1500,  -1436,
-         -1372,  -1308,  -1244,  -1180,  -1116,  -1052,   -988,   -924,
-          -876,   -844,   -812,   -780,   -748,   -716,   -684,   -652,
-          -620,   -588,   -556,   -524,   -492,   -460,   -428,   -396,
-          -372,   -356,   -340,   -324,   -308,   -292,   -276,   -260,
-          -244,   -228,   -212,   -196,   -180,   -164,   -148,   -132,
-          -120,   -112,   -104,    -96,    -88,    -80,    -72,    -64,
-           -56,    -48,    -40,    -32,    -24,    -16,     -8,      0,
-         32124,  31100,  30076,  29052,  28028,  27004,  25980,  24956,
-         23932,  22908,  21884,  20860,  19836,  18812,  17788,  16764,
-         15996,  15484,  14972,  14460,  13948,  13436,  12924,  12412,
-         11900,  11388,  10876,  10364,   9852,   9340,   8828,   8316,
-          7932,   7676,   7420,   7164,   6908,   6652,   6396,   6140,
-          5884,   5628,   5372,   5116,   4860,   4604,   4348,   4092,
-          3900,   3772,   3644,   3516,   3388,   3260,   3132,   3004,
-          2876,   2748,   2620,   2492,   2364,   2236,   2108,   1980,
-          1884,   1820,   1756,   1692,   1628,   1564,   1500,   1436,
-          1372,   1308,   1244,   1180,   1116,   1052,    988,    924,
-           876,    844,    812,    780,    748,    716,    684,    652,
-           620,    588,    556,    524,    492,    460,    428,    396,
-           372,    356,    340,    324,    308,    292,    276,    260,
-           244,    228,    212,    196,    180,    164,    148,    132,
-           120,    112,    104,     96,     88,     80,     72,     64,
-            56,     48,     40,     32,     24,     16,      8,      0
-       };
-
 struct Sound_recordFixedTime_Info {
 	integer numberOfSamples, numberOfSamplesRead;
 	short *buffer;
@@ -143,12 +108,10 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 	#endif
 	try {
 		integer numberOfSamples, i;
-		bool mulaw = false;
-		bool can16bit = true;
-		bool fakeMonoByStereo = false;   // will be set to `true` for systems (like MacOS X) that do not allow direct mono recording
-
-		/* Declare system-dependent data structures. */
 
+		/*
+			Declare platform-dependent data structures.
+		*/
 		volatile struct Sound_recordFixedTime_Info info = { 0 };
 		PaStreamParameters streamParameters = { 0 };
 		#if defined (macintosh)
@@ -167,13 +130,15 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			int val;
 		#endif
 
-		/* Check representation of shorts. */
-
+		/*
+			Check representation of shorts.
+		*/
 		if (sizeof (short) != 2)
 			Melder_throw (U"Cannot record a sound on this computer.");
 
-		/* Check sampling frequency. */
-
+		/*
+			Check sampling frequency.
+		*/
 		bool supportsSamplingFrequency = true;
 		if (inputUsesPortAudio) {
 			#if defined (macintosh)
@@ -202,10 +167,10 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			Melder_throw (U"Your audio hardware does not support a sampling frequency of ", sampleRate, U" Hz.");
 
 		/*
-		 * Open phase 1.
-		 * On some systems, the info is filled in before the audio port is opened.
-		 * On other systems, the info is filled in after the port is opened.
-		 */
+			Open phase 1.
+			On some platforms, the info is filled in before the audio port is opened.
+			On other platforms, the info is filled in after the port is opened.
+		*/
 		if (inputUsesPortAudio) {
 			if (! MelderAudio_hasBeenInitialized) {
 				PaError err = Pa_Initialize ();
@@ -217,7 +182,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#if defined (macintosh)
 			#elif defined (_WIN32)
 			#elif ! defined (NO_AUDIO)
-				/* We must open the port now, because we use an ioctl to set the info to an open port. */
+				/*
+					We must open the port now, because we use an ioctl to set the info to an open port.
+				*/
 				fd = open (DEV_AUDIO, O_RDONLY);
 				if (fd == -1) {
 					if (errno == EBUSY)
@@ -229,20 +196,30 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 							Melder_throw (U"Cannot open audio device.");
 						#endif
 				}
-				/* The device immediately started recording into its buffer, but probably at the wrong rate etc. */
-				/* Pause and flush this rubbish. */
+				/*
+					The device immediately started recording into its buffer,
+					but probably at the wrong rate etc.
+					Pause and flush this rubbish.
+				*/
 				#if defined (linux)
 					ioctl (fd, SNDCTL_DSP_RESET, nullptr);
 				#endif
 			#endif
 		}
 
-		/* Set the input source; the default is the microphone. */
-
+		/*
+			Set the input source; the default is the microphone.
+		*/
 		if (inputUsesPortAudio) {
 			if (inputSource < 1 || inputSource > Pa_GetDeviceCount ())
 				Melder_throw (U"Unknown device #", inputSource, U".");
-			streamParameters. device = inputSource - 1;
+			/*
+				Saying
+					streamParameters. device = inputSource - 1;
+				would presuppose that the input devices are listed before the output devices.
+				TODO: cycle through all devices, and determine which of them are input devices
+			*/
+			streamParameters. device = Pa_GetDefaultInputDevice ();
 		} else {
 			#if defined (macintosh)
 			#elif defined (linux) && ! defined (NO_AUDIO)
@@ -255,8 +232,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#endif
 		}
 
-		/* Set gain and balance. */
-
+		/*
+			Set gain and balance.
+		*/
 		if (inputUsesPortAudio) {
 			/* Taken from Audio Control Panel. */
 		} else {
@@ -285,8 +263,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#endif
 		}
 
-		/* Set the sampling frequency. */
-
+		/*
+			Set the sampling frequency.
+		*/
 		if (inputUsesPortAudio) {
 			// Set while opening.
 		} else {
@@ -300,8 +279,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#endif
 		}
 
-		/* Set the number of channels to 1 (mono), if possible. */
-
+		/*
+			Set the number of channels to 1 (mono), if possible.
+		*/
 		if (inputUsesPortAudio) {
 			streamParameters. channelCount = 1;
 		} else {
@@ -315,8 +295,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#endif
 		}
 
-		/* Set the encoding to 16-bit linear (or to 8-bit linear, if 16-bit is not available). */
-
+		/*
+			Set the encoding to 16-bit linear (or to 8-bit linear, if 16-bit is not available).
+		*/
 		if (inputUsesPortAudio) {
 			streamParameters. sampleFormat = paInt16;
 		} else {
@@ -337,22 +318,22 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 			#endif
 		}
 
-		/* Create a buffer for recording, and the resulting sound. */
-
+		/*
+			Create a buffer for recording, and the resulting sound.
+		*/
 		numberOfSamples = Melder_iround (sampleRate * duration);
 		if (numberOfSamples < 1)
 			Melder_throw (U"Duration too short.");
-		autovector<short> buffer = newvectorzero<short> (numberOfSamples * (fakeMonoByStereo ? 2 : 1));
-		autoSound me = Sound_createSimple (1, numberOfSamples / sampleRate, sampleRate);   // STEREO BUG
+		autovector<short> buffer = newvectorzero <short> (numberOfSamples);
+		autoSound me = Sound_createSimple (1, numberOfSamples / sampleRate, sampleRate);
 		Melder_assert (my nx == numberOfSamples);
 
 		/*
-		 * Open phase 2.
-		 * This starts recording now.
-		 */
-
+			Open phase 2.
+			This starts recording now.
+		*/
 		if (inputUsesPortAudio) {
-			streamParameters. suggestedLatency = Pa_GetDeviceInfo (inputSource - 1) -> defaultLowInputLatency;
+			streamParameters. suggestedLatency = Pa_GetDeviceInfo (streamParameters. device) -> defaultLowInputLatency;
 			#if defined (macintosh)
 				PaMacCoreStreamInfo macCoreStreamInfo = { 0 };
 				macCoreStreamInfo. size = sizeof (PaMacCoreStreamInfo);
@@ -382,8 +363,9 @@ autoSound Sound_record_fixedTime (int inputSource, double gain, double balance,
 		}
 for (i = 1; i <= numberOfSamples; i ++) trace (U"Started ", buffer [i]);
 
-		/* Read the sound into the buffer. */
-
+		/*
+			Read the sound into the buffer.
+		*/
 		if (inputUsesPortAudio) {
 			// The callback will do this. Just wait.
 			while (/*getNumberOfSamplesRead (& info)*/ info. numberOfSamplesRead < numberOfSamples) {
@@ -414,39 +396,26 @@ for (i = 1; i <= numberOfSamples; i ++) trace (U"Recorded ", buffer [i]);
 				if (err != MMSYSERR_NOERROR)
 					Melder_throw (U"Error ", err, U" while unpreparing header.");
 			#else
-				if (mulaw)
-					read (fd, (char *) & buffer [1], numberOfSamples);
-				else {
-					integer bytesLeft = 2 * numberOfSamples, dbytes, bytesRead = 0;
-					while (bytesLeft) {
-						//Melder_casual ("Reading %ld bytes", bytesLeft > 4000 ? 4000 : bytesLeft);
-						dbytes = read (fd, (char *) & buffer [2 + bytesRead], bytesLeft > 4000 ? 4000 : bytesLeft);
-						//Melder_casual("Read %ld bytes", dbytes);
-						if (dbytes <= 0) break;
-						bytesLeft -= dbytes;
-						bytesRead += dbytes;
-					};
-				}
+				integer bytesLeft = 2 * numberOfSamples, dbytes, bytesRead = 0;
+				while (bytesLeft) {
+					dbytes = read (fd, (char *) & buffer [2 + bytesRead], std::min (bytesLeft, 4000_integer));
+					if (dbytes <= 0)
+						break;
+					bytesLeft -= dbytes;
+					bytesRead += dbytes;
+				};
 			#endif
 		}
 
-		/* Copy the buffered data to the sound object, and discard the buffer. */
-
-		if (fakeMonoByStereo)
-			for (i = 1; i <= numberOfSamples; i ++)
-				my z [1] [i] = ((integer) buffer [i + i - 1] + buffer [i + i]) * (1.0 / 65536);
-		else if (mulaw)
-			for (i = 1; i <= numberOfSamples; i ++)
-				my z [1] [i] = ulaw2linear [((unsigned char *) buffer.begin()) [i]] * (1.0 / 32768);
-		else if (can16bit)
-			for (i = 1; i <= numberOfSamples; i ++)
-				my z [1] [i] = buffer [i] * (1.0 / 32768);
-		else
-			for (i = 1; i <= numberOfSamples; i ++)
-				my z [1] [i] = ((int) ((unsigned char *) buffer.begin()) [i + 1] - 128) * (1.0 / 128);
-
-		/* Close the audio device. */
+		/*
+			Copy the buffered data to the sound object, and discard the buffer.
+		*/
+		for (i = 1; i <= numberOfSamples; i ++)
+			my z [1] [i] = buffer [i] * (1.0 / 32768);
 
+		/*
+			Close the audio device.
+		*/
 		if (inputUsesPortAudio) {
 			Pa_StopStream (portaudioStream);
 			Pa_CloseStream (portaudioStream);
@@ -461,8 +430,9 @@ for (i = 1; i <= numberOfSamples; i ++) trace (U"Recorded ", buffer [i]);
 			#endif
 		}
 
-		/* Hand the resulting sound to the caller. */
-
+		/*
+			Hand the resulting sound to the caller.
+		*/
 		return me;
 	} catch (MelderError) {
 		if (inputUsesPortAudio) {
