@@ -1,6 +1,6 @@
 /* Spectrogram.cpp
  *
- * Copyright (C) 1992-2008,2011,2012,2015-2018 Paul Boersma
+ * Copyright (C) 1992-2008,2011,2012,2015-2018,2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,14 +56,20 @@ void Spectrogram_paintInside (Spectrogram me, Graphics g, double tmin, double tm
 	double maximum, int autoscaling, double dynamic, double preemphasis, double dynamicCompression)
 {
 	Function_unidirectionalAutowindow (me, & tmin, & tmax);
-	if (fmax <= fmin) { fmin = my ymin; fmax = my ymax; }
+	if (fmax <= fmin) {
+		fmin = my ymin;
+		fmax = my ymax;
+	}
 	integer itmin, itmax, ifmin, ifmax;
-	if (Matrix_getWindowSamplesX (me, tmin - 0.49999 * my dx, tmax + 0.49999 * my dx, & itmin, & itmax) == 0||
-			Matrix_getWindowSamplesY (me, fmin - 0.49999 * my dy, fmax + 0.49999 * my dy, & ifmin, & ifmax) == 0)
+	const auto nt = Matrix_getWindowSamplesX (me, tmin - 0.49999 * my dx, tmax + 0.49999 * my dx, & itmin, & itmax);
+	const auto nf = Matrix_getWindowSamplesY (me, fmin - 0.49999 * my dy, fmax + 0.49999 * my dy, & ifmin, & ifmax);
+	if (nt == 0 || nf == 0)
 		return;
 	Graphics_setWindow (g, tmin, tmax, fmin, fmax);
-	autoNUMvector <double> preemphasisFactor (ifmin, ifmax);
-	autoNUMvector <double> dynamicFactor (itmin, itmax);
+	auto preemphasisFactorBuffer = newVECzero (nf);
+	double *preemphasisFactor = & preemphasisFactorBuffer [1 - ifmin];
+	auto dynamicFactorBuffer = newVECzero (nt);
+	double *dynamicFactor = & dynamicFactorBuffer [1 - itmin];
 	/* Pre-emphasis in place; also compute maximum after pre-emphasis. */
 	for (integer ifreq = ifmin; ifreq <= ifmax; ifreq ++) {
 		preemphasisFactor [ifreq] = (preemphasis / NUMln2) * log (ifreq * my dy / 1000.0);
