@@ -1,6 +1,6 @@
 /* Net.cpp
  *
- * Copyright (C) 2017-2019 Paul Boersma
+ * Copyright (C) 2017-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -475,7 +475,7 @@ autoActivationList Net_PatternList_to_ActivationList (Net me, PatternList thee,
 		for (integer ipattern = 1; ipattern <= thy ny; ipattern ++) {
 			Net_PatternList_applyToInput (me, thee, ipattern);
 			Net_spreadUp (me, activationType);
-			activations -> z.row (ipattern) <<= outputLayer -> outputActivities;
+			activations -> z.row (ipattern) <<= outputLayer -> outputActivities.all();
 		}
 		return activations;
 	} catch (MelderError) {
@@ -486,7 +486,7 @@ autoActivationList Net_PatternList_to_ActivationList (Net me, PatternList thee,
 static autoMatrix Layer_extractInputActivities (Layer me) {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, my numberOfInputNodes);
-		thy z.row (1) <<= my inputActivities;
+		thy z.row (1) <<= my inputActivities.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": input activities not extracted.");
@@ -500,7 +500,7 @@ autoMatrix Net_extractInputActivities (Net me) {
 static autoMatrix Layer_extractOutputActivities (Layer me) {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, my numberOfOutputNodes);
-		thy z.row (1) <<= my outputActivities;
+		thy z.row (1) <<= my outputActivities.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (me, U": output activities not extracted.");
@@ -514,7 +514,7 @@ autoMatrix Net_extractOutputActivities (Net me) {
 autoMatrix structRBMLayer :: v_extractInputReconstruction () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfInputNodes);
-		thy z.row (1) <<= our inputReconstruction;
+		thy z.row (1) <<= our inputReconstruction.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input reconstruction not extracted.");
@@ -528,7 +528,7 @@ autoMatrix Net_extractInputReconstruction (Net me) {
 autoMatrix structRBMLayer :: v_extractOutputReconstruction () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfOutputNodes);
-		thy z.row (1) <<= our outputReconstruction;
+		thy z.row (1) <<= our outputReconstruction.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": output reconstruction not extracted.");
@@ -542,7 +542,7 @@ autoMatrix Net_extractOutputReconstruction (Net me) {
 autoMatrix structRBMLayer :: v_extractInputBiases () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfInputNodes);
-		thy z.row (1) <<= our inputBiases;
+		thy z.row (1) <<= our inputBiases.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input biases not extracted.");
@@ -568,7 +568,7 @@ autoMatrix Net_extractInputBiases (Net me, integer layerNumber) {
 autoMatrix structRBMLayer :: v_extractOutputBiases () {
 	try {
 		autoMatrix thee = Matrix_createSimple (1, our numberOfOutputNodes);
-		thy z.row (1) <<= our outputBiases;
+		thy z.row (1) <<= our outputBiases.all();
 		return thee;
 	} catch (MelderError) {
 		Melder_throw (this, U": input biases not extracted.");
