@@ -2,7 +2,7 @@
 #define _melder_arg_h_
 /* MelderArg.h
  *
- * Copyright (C) 1992-2018 Paul Boersma
+ * Copyright (C) 1992-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@ struct MelderArg {
 	*/
 	MelderArg (constVECVU const&         arg) : _arg (Melder_VEC             (arg)) { }
 	MelderArg (constMATVU const&         arg) : _arg (Melder_MAT             (arg)) { }
+	MelderArg (VECVU const&              arg) : _arg (Melder_VEC             (arg)) { }
+	MelderArg (MATVU const&              arg) : _arg (Melder_MAT             (arg)) { }
 	MelderArg (Thing                     arg) : _arg (Thing_messageName      (arg)) { }
 	MelderArg (MelderFile                arg) : _arg (MelderFile_messageName (arg)) { }
 	/*
