@@ -24,7 +24,6 @@
 /* These functions also maintain a count of the total number of blocks allocated. */
 
 void Melder_alloc_init ();   // to be called around program start-up
-void Melder_message_init ();   // to be called around program start-up
 void * _Melder_malloc (int64 size);
 #define Melder_malloc(type,numberOfElements)  (type *) _Melder_malloc ((numberOfElements) * (int64) sizeof (type))
 void * _Melder_malloc_f (int64 size);
@@ -68,5 +67,33 @@ int64 Melder_allocationSize ();
 int64 Melder_reallocationsInSituCount ();
 int64 Melder_movingReallocationsCount ();
 
+/********** Arrays. **********/
+
+
+namespace MelderArray {
+
+	enum class kInitializationType { RAW = 0, ZERO = 1 };
+
+	byte * _alloc_generic (integer cellSize, integer numberOfCells, kInitializationType initializationType);
+	void _free_generic (byte *cells, integer numberOfCells) noexcept;
+
+	template <class T>
+	T* _alloc (integer numberOfCells, kInitializationType initializationType) {
+		T* result = reinterpret_cast <T*> (MelderArray:: _alloc_generic (sizeof (T), numberOfCells, initializationType));
+		return result;
+	}
+
+	template <class T>
+	void _free (T* cells, integer numberOfCells) noexcept {
+		_free_generic (reinterpret_cast <byte *> (cells), numberOfCells);
+	}
+
+}
+
+int64 MelderArray_allocationCount ();
+int64 MelderArray_deallocationCount ();
+int64 MelderArray_cellAllocationCount ();
+int64 MelderArray_cellDeallocationCount ();
+
 /* End of file melder_alloc.h */
 #endif
