@@ -1,6 +1,6 @@
 /* melder_debug.cpp
  *
- * Copyright (C) 2000-2018 Paul Boersma
+ * Copyright (C) 2000-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,6 @@ the behaviour of Praat will temporarily change in the following ways:
 34: trace memory allocation and deallocation
 35: debugging on for QuickTime movie file opening
 37: debug autoThing
-38: debug autoNUMvector
 39: debug autostring
 40: debug Thing_new
 41: OTGrammar_getWinner: always first choice rather than random choice 
@@ -82,6 +81,7 @@ the behaviour of Praat will temporarily change in the following ways:
 50: compute sum, mean, stdev with first-element offset (80 bits)
 51: compute sum, mean, stdev with two cycles, as in R (80 bits)
 (other numbers than 48-51: compute sum, mean, stdev with simple pairwise algorithm, base case 64 [80 bits])
+52: debug Discriminant_TableOfReal_to_ClassificationTable
 181: read and write native-endian real64
 900: use DG Meta Serif Science instead of Palatino
 1264: Mac: Sound_record_fixedTime uses microphone "FW Solo (1264)"
