@@ -1,6 +1,6 @@
 /* Table.cpp
  *
- * Copyright (C) 2002-2019 Paul Boersma
+ * Copyright (C) 2002-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
  * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <ctype.h>
 #include "Table.h"
 #include "NUM2.h"
 #include "Formula.h"
@@ -182,11 +181,11 @@ void Table_removeColumn (Table me, integer columnNumber) {
 			Changes without error.
 		*/
 		my columnHeaders [columnNumber]. destroy ();
-		my columnHeaders.remove (columnNumber);
+		my columnHeaders. remove (columnNumber);
 		for (integer irow = 1; irow <= my rows.size; irow ++) {
 			TableRow row = my rows.at [irow];
 			row -> cells [columnNumber]. destroy ();
-			row -> cells.remove (columnNumber);
+			row -> cells. remove (columnNumber);
 			row -> numberOfColumns --;   // maintain invariant
 		}
 		my numberOfColumns --;   // maintain invariant
@@ -770,7 +769,7 @@ autoTable Table_collapseRows (Table me, conststring32 factors_string, conststrin
 		/*
 			We will now sort the original table temporarily, by the factors (independent variables) only.
 		*/
-		Table_sortRows_Assert (me, constINTVEC (columns.at, factors.size));   /* This works only because the factors come first. */
+		Table_sortRows_Assert (me, constINTVEC (columns.cells, factors.size));   /* This works only because the factors come first. */
 		originalChanged = true;
 		/*
 			Find stretches of identical factors.
@@ -881,7 +880,7 @@ static autoSTRVEC Table_getLevels_ (Table me, integer column) {
 			TableRow row = my rows.at [irow];
 			row -> sortingIndex = irow;
 		}
-		integer columns [2] = { 0, column };
+		integer columns [1] = { column };
 		Table_sortRows_Assert (me, constINTVEC (columns, 1));
 		integer numberOfLevels = 0;
 		integer irow = 1;
@@ -1060,7 +1059,7 @@ static int cellCompare (const void *first, const void *second) {
 	const TableRow me = * (TableRow *) first, thee = * (TableRow *) second;
 	const integer ncol = cellCompare_columns->size;
 	for (integer icol = 1; icol <= ncol; icol ++) {
-		const integer cellNumber = cellCompare_columns->at [icol];
+		const integer cellNumber = (*cellCompare_columns) [icol];
 		if (my cells [cellNumber]. number < thy cells [cellNumber]. number)
 			return -1;
 		if (my cells [cellNumber]. number > thy cells [cellNumber]. number)
@@ -1669,7 +1668,7 @@ double Table_getGroupDifference_wilcoxonRankSum (Table me, integer column, integ
 	Table_numericize_Assert (ranks.get(), 1);
 	Table_numericize_Assert (ranks.get(), 2);
 	Table_numericize_Assert (ranks.get(), 3);
-	integer columns [1+1] = { 0, 2 };   // we're gonna sort by column 2
+	integer columns [1] = { 2 };   // we're gonna sort by column 2
 	Table_sortRows_Assert (ranks.get(), constINTVEC (columns, 1));   // we sort by one column only
 	double totalNumberOfTies3 = 0.0;
 	for (integer irow = 1; irow <= ranks -> rows.size; irow ++) {
@@ -1728,6 +1727,25 @@ bool Table_getExtrema (Table me, integer icol, double *minimum, double *maximum)
 	return true;
 }
 
+/*@praat
+	#
+	# Paul Boersma 2002-05-02
+	#
+	# bug reported by Michelle Porter on 2002-05-01
+	#
+
+	Create Table with column names: "table", 4, "A B"
+	Set numeric value: 1, "A", 1
+	Set numeric value: 2, "A", 2
+	Set numeric value: 3, "A", 3
+	Set numeric value: 4, "A", 4
+	Set numeric value: 1, "B", 4
+	Set numeric value: 2, "B", 3
+	Set numeric value: 3, "B", 2
+	Set numeric value: 4, "B", 1
+	Scatter plot (mark): "A", 0, 0, "B", 0, 0, 1, "yes", "+"
+	Remove
+@*/
 void Table_scatterPlot_mark (Table me, Graphics g, integer xcolumn, integer ycolumn,
 	double xmin, double xmax, double ymin, double ymax, double markSize_mm, conststring32 mark, bool garnish)
 {
@@ -2021,7 +2039,7 @@ autoTable Table_readFromTableFile (MelderFile file) {
 				Melder_assert (*p != U'\0');
 				p ++;
 			}
-			static MelderString buffer { };
+			static MelderString buffer;
 			MelderString_empty (& buffer);
 			while (*p != U' ' && *p != U'\t' && *p != U'\n') {
 				MelderString_appendCharacter (& buffer, *p);
@@ -2034,7 +2052,7 @@ autoTable Table_readFromTableFile (MelderFile file) {
 			TableRow row = my rows.at [irow];
 			for (integer icol = 1; icol <= numberOfColumns; icol ++) {
 				while (*p == U' ' || *p == U'\t' || *p == U'\n') { Melder_assert (*p != U'\0'); p ++; }
-				static MelderString buffer { };
+				static MelderString buffer;
 				MelderString_empty (& buffer);
 				while (*p != U' ' && *p != U'\t' && *p != U'\n' && *p != U'\0') {
 					MelderString_appendCharacter (& buffer, *p);
