@@ -1,6 +1,6 @@
 /* Thing.cpp
  *
- * Copyright (C) 1992-2012,2015,2017,2018 Paul Boersma
+ * Copyright (C) 1992-2012,2014-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,13 +67,13 @@ static void _Thing_addOneReadableClass (ClassInfo readableClass) {
 }
 void Thing_recognizeClassesByName (ClassInfo readableClass, ...) {
 	va_list arg;
-	if (! readableClass) return;
+	if (! readableClass)
+		return;
 	va_start (arg, readableClass);
 	_Thing_addOneReadableClass (readableClass);
 	ClassInfo klas;
-	while ((klas = va_arg (arg, ClassInfo)) != nullptr) {
+	while ((klas = va_arg (arg, ClassInfo)) != nullptr)
 		_Thing_addOneReadableClass (klas);
-	}
 	va_end (arg);
 }
 
@@ -106,24 +106,25 @@ ClassInfo Thing_classFromClassName (conststring32 klas, int *out_formatVersion)
 	char32 *space = str32chr (buffer, U' ');
 	if (space) {
 		*space = U'\0';   // strip version number
-		if (out_formatVersion) *out_formatVersion = Melder_atoi (space + 1);
+		if (out_formatVersion)
+			*out_formatVersion = Melder_atoi (space + 1);
 	} else {
-		if (out_formatVersion) *out_formatVersion = 0;
+		if (out_formatVersion)
+			*out_formatVersion = 0;
 	}
 
 	/*
-	 * First try the class names that were registered with Thing_recognizeClassesByName.
-	 */
+		First try the class names that were registered with Thing_recognizeClassesByName.
+	*/
 	for (integer i = 1; i <= theNumberOfReadableClasses; i ++) {
 		ClassInfo classInfo = theReadableClasses [i];
-		if (str32equ (buffer, classInfo -> className)) {
+		if (str32equ (buffer, classInfo -> className))
 			return classInfo;
-		}
 	}
 
 	/*
-	 * Then try the aliases that were registered with Thing_recognizeClassByOtherName.
-	 */
+		Then try the aliases that were registered with Thing_recognizeClassByOtherName.
+	*/
 	for (integer i = 1; i <= theNumberOfAliases; i ++) {
 		if (str32equ (buffer, theAliases [i]. otherName)) {
 			ClassInfo classInfo = theAliases [i]. readableClass;
@@ -144,16 +145,17 @@ autoThing Thing_newFromClassName (conststring32 className, int *out_formatVersio
 }
 
 Thing _Thing_dummyObject (ClassInfo classInfo) {
-	if (! classInfo -> dummyObject) {
+	if (! classInfo -> dummyObject)
 		classInfo -> dummyObject = classInfo -> _new ();
-	}
 	Melder_assert (classInfo -> dummyObject);
 	return classInfo -> dummyObject;
 }
 
 void _Thing_forget_nozero (Thing me) {
-	if (! me) return;
-	if (Melder_debug == 40) Melder_casual (U"destroying ", my classInfo -> className);
+	if (! me)
+		return;
+	if (Melder_debug == 40)
+		Melder_casual (U"destroying ", my classInfo -> className);
 	//Melder_casual (U"_Thing_forget_nozero before");
 	my v_destroy ();
 	//Melder_casual (U"_Thing_forget_nozero after");
@@ -162,7 +164,8 @@ void _Thing_forget_nozero (Thing me) {
 
 void _Thing_forget (Thing me) {
 	if (! me) return;
-	if (Melder_debug == 40) Melder_casual (U"destroying ", my classInfo -> className);
+	if (Melder_debug == 40)
+		Melder_casual (U"destroying ", my classInfo -> className);
 	my v_destroy ();
 	trace (U"destroyed ", my classInfo -> className, U" ", Melder_pointer (me));
 	//Melder_free (me);
@@ -172,12 +175,14 @@ void _Thing_forget (Thing me) {
 }
 
 bool Thing_isSubclass (ClassInfo klas, ClassInfo ancestor) {
-	while (klas != ancestor && klas) klas = klas -> semanticParent;
+	while (klas != ancestor && klas)
+		klas = klas -> semanticParent;
 	return !! klas;
 }
 
 bool Thing_isa (Thing me, ClassInfo klas) {
-	if (! me) Melder_fatal (U"(Thing_isa:) Found null object.");
+	if (! me)
+		Melder_fatal (U"(Thing_isa:) Found null object.");
 	return Thing_isSubclass (my classInfo, klas);
 }
 
@@ -186,7 +191,8 @@ void Thing_infoWithIdAndFile (Thing me, integer id, MelderFile file) {
 	Melder_clearInfo ();
 	MelderInfo_open ();
 	if (id != 0) MelderInfo_writeLine (U"Object id: ", id);
-	if (! MelderFile_isNull (file)) MelderInfo_writeLine (U"Associated file: ", Melder_fileToPath (file));
+	if (! MelderFile_isNull (file))
+		MelderInfo_writeLine (U"Associated file: ", Melder_fileToPath (file));
 	my v_info ();
 	MelderInfo_close ();
 }
@@ -198,14 +204,14 @@ void Thing_info (Thing me) {
 conststring32 Thing_getName (Thing me) { return my name.get(); }
 
 conststring32 Thing_messageName (Thing me) {
-	static MelderString buffers [19] { };
+	static MelderString buffers [19];
 	static int ibuffer = 0;
-	if (++ ibuffer == 19) ibuffer = 0;
-	if (my name) {
+	if (++ ibuffer == 19)
+		ibuffer = 0;
+	if (my name)
 		MelderString_copy (& buffers [ibuffer], my classInfo -> className, U" \"", my name.get(), U"\"");
-	} else {
+	else
 		MelderString_copy (& buffers [ibuffer], my classInfo -> className);
-	}
 	return buffers [ibuffer]. string;
 }
 
