@@ -1,6 +1,6 @@
 /* praat.cpp
  *
- * Copyright (C) 1992-2019 Paul Boersma
+ * Copyright (C) 1992-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
  */
 
 #include "melder.h"
-#include <ctype.h>
 #include <stdarg.h>
 #if defined (UNIX) || defined (macintosh)
 	#include <sys/types.h>
@@ -137,10 +136,10 @@ integer praat_idOfSelected (ClassInfo klas, integer inplace) {
 }
 
 autoVEC praat_idsOfAllSelected (ClassInfo klas) {
-	autoVEC result (praat_numberOfSelected (klas), kTensorInitializationType::RAW);
+	autoVEC result = newVECraw (praat_numberOfSelected (klas));
 	integer selectedObjectNumber = 0, IOBJECT;
 	WHERE (SELECTED && (! klas || CLASS == klas)) {
-		result.at [++ selectedObjectNumber] = ID;
+		result [++ selectedObjectNumber] = ID;
 	}
 	return result;
 }
@@ -197,24 +196,24 @@ void praat_select (int IOBJECT) {
 	Thing object = theCurrentPraatObjects -> list [IOBJECT]. object;
 	Melder_assert (object);
 	integer readableClassId = object -> classInfo -> sequentialUniqueIdOfReadableClass;
-	if (readableClassId == 0) Melder_fatal (U"No sequential unique ID for class ", object -> classInfo -> className, U".");
+	if (readableClassId == 0)
+		Melder_fatal (U"No sequential unique ID for class ", object -> classInfo -> className, U".");
 	theCurrentPraatObjects -> numberOfSelected [readableClassId] += 1;
-	if (! theCurrentPraatApplication -> batch && ! Melder_backgrounding) {
+	if (! theCurrentPraatApplication -> batch && ! Melder_backgrounding)
 		GuiList_selectItem (praatList_objects, IOBJECT);
-	}
 }
 
 void praat_selectAll () { int IOBJECT; WHERE (1) praat_select (IOBJECT); }
 
 void praat_list_background () {
 	int IOBJECT;
-	WHERE (SELECTED) GuiList_deselectItem (praatList_objects, IOBJECT);
+	WHERE (SELECTED)
+		GuiList_deselectItem (praatList_objects, IOBJECT);
 }
 void praat_list_foreground () {
 	int IOBJECT;
-	WHERE (SELECTED) {
+	WHERE (SELECTED)
 		GuiList_selectItem (praatList_objects, IOBJECT);
-	}
 }
 
 autoCollection praat_getSelectedObjects () {
@@ -230,8 +229,8 @@ autoCollection praat_getSelectedObjects () {
 char32 *praat_name (int IOBJECT) { return str32chr (FULL_NAME, U' ') + 1; }
 
 void praat_write_do (UiForm dia, conststring32 extension) {
-	static MelderString defaultFileName { };
-	if (extension && str32chr (extension, '.')) {
+	static MelderString defaultFileName;
+	if (extension && str32chr (extension, U'.')) {
 		/*
 			Apparently, the "extension" is a complete file name.
 			This should be used as the default file name.
@@ -245,10 +244,17 @@ void praat_write_do (UiForm dia, conststring32 extension) {
 		*/
 		int IOBJECT, found = 0;
 		Daata data = nullptr;
-		WHERE (SELECTED) { if (! data) data = (Daata) OBJECT; found += 1; }
+		WHERE (SELECTED) {
+			if (! data)
+				data = (Daata) OBJECT;
+			found += 1;
+		}
 		if (found == 1) {
 			MelderString_copy (& defaultFileName, data -> name.get());
-			if (defaultFileName.length > 200) { defaultFileName.string [200] = U'\0'; defaultFileName.length = 200; }
+			if (defaultFileName.length > 200) {
+				defaultFileName.string [200] = U'\0';
+				defaultFileName.length = 200;
+			}
 			MelderString_append (& defaultFileName, U".", extension ? extension : Thing_className (data));
 		} else if (! extension) {
 			MelderString_copy (& defaultFileName, U"praat.Collection");
@@ -389,7 +395,7 @@ void praat_newWithFile (autoDaata me, MelderFile file, conststring32 myName) {
 	theCurrentPraatObjects -> totalBeingCreated ++;
 }
 
-static MelderString thePraatNewName { };
+static MelderString thePraatNewName;
 void praat_new (autoDaata me) {
 	praat_newWithFile (me.move(), nullptr, U"");
 }
@@ -1404,9 +1410,8 @@ void praat_init (conststring32 title, int argc, char **argv)
 			}
 		#endif
 		#ifdef UNIX
-			if (! praatP.ignorePreferenceFiles) {
+			if (! praatP.ignorePreferenceFiles)
 				Preferences_read (& prefsFile);
-			}
 		#endif
 		#if ! defined (macintosh)
 			trace (U"initializing the Gui late");
@@ -1502,7 +1507,8 @@ void praat_run () {
 	 */
 	if (! praatP.ignorePreferenceFiles) {
 		Preferences_read (& prefsFile);
-		if (! praatP.dontUsePictureWindow) praat_picture_prefsChanged ();
+		if (! praatP.dontUsePictureWindow)
+			praat_picture_prefsChanged ();
 		praat_statistics_prefsChanged ();
 	}
 
@@ -1727,16 +1733,16 @@ void praat_run () {
 	Melder_assert (str32equ (Melder_colour (MelderColour (0.25, 0.50, 0.875)), U"{0.25,0.5,0.875}"));
 	{
 		VEC xn;   // uninitialized
-		Melder_assert (! xn.at);
+		Melder_assert (! xn.cells);
 		Melder_assert (xn.size == 0);
 		VEC x { };
-		Melder_assert (! x.at);
+		Melder_assert (! x.cells);
 		Melder_assert (x.size == 0);
 		constVEC xnc;   // zero-initialized
-		Melder_assert (! xnc.at);
+		Melder_assert (! xnc.cells);
 		Melder_assert (xnc.size == 0);
 		constVEC xc { };
-		Melder_assert (! xc.at);
+		Melder_assert (! xc.cells);
 		Melder_assert (xc.size == 0);
 		MAT yn;
 		Melder_assert (! yn.cells);
@@ -1819,7 +1825,8 @@ void praat_run () {
 						} catch (MelderError) {
 							Melder_clearError ();   // ignore this line, but not necessarily the next
 						}
-						if (! newline) break;
+						if (! newline)
+							break;
 						line = newline + 1;
 					}
 				}
