@@ -1,6 +1,6 @@
 /* praat_objectMenus.cpp
  *
- * Copyright (C) 1992-2019 Paul Boersma
+ * Copyright (C) 1992-2020 Paul Boersma
  *
  * This code is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
  * along with this work. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <ctype.h>
 #include "praatP.h"
 #include "praat_script.h"
 #include "ScriptEditor.h"
@@ -52,7 +51,7 @@ DO
 	static MelderString string;
 	MelderString_copy (& string, newName);
 	praat_cleanUpName (string.string);
-	static MelderString fullName { };
+	static MelderString fullName;
 	MelderString_copy (& fullName, Thing_className (OBJECT), U" ", string.string);
 	if (! str32equ (fullName.string, FULL_NAME)) {
 		theCurrentPraatObjects -> list [IOBJECT]. name = Melder_dup_f (fullName.string);
@@ -520,13 +519,13 @@ END }
 FORM (HELP_GoToManualPage, U"Go to manual page", nullptr) {
 	static constSTRVEC pages;
 	pages = ManPages_getTitles (theCurrentPraatApplication -> manPages);
-	LIST (pageNumber, U"Page", pages, 1)
+	LIST (goToPageNumber, U"Page", pages, 1)
 	OK
 DO
 	if (theCurrentPraatApplication -> batch)
 		Melder_throw (U"Cannot view a manual from batch.");
 	autoManual manual = Manual_create (U"Intro", theCurrentPraatApplication -> manPages, false);
-	HyperPage_goToPage_i (manual.get(), pageNumber);
+	HyperPage_goToPage_number (manual.get(), goToPageNumber);
 	manual.releaseToUser();
 END }
 
@@ -570,7 +569,7 @@ static void searchProc () {
 	HELP_SearchManual (nullptr, 0, nullptr, nullptr, nullptr, nullptr, false, nullptr);
 }
 
-static MelderString itemTitle_about { };
+static MelderString itemTitle_about;
 
 static autoDaata scriptRecognizer (integer nread, const char *header, MelderFile file) {
 	conststring32 name = MelderFile_name (file);
